#!/usr/bin/python

"""
 Copyright (c) 2009
      Oracle Corporation.  All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Redistributions in any form must be accompanied by information on
    how to obtain complete source code for the Oracle VM CLI and any
    accompanying software that uses the Oracle VM CLI.  The source code
    must either be included in the distribution or be available for no
    more than the cost of distribution plus a nominal fee, and must be
    freely redistributable under reasonable conditions.  For an
    executable file, complete source code means the source code for all
    modules it contains.  It does not include source code for modules or
    files that typically accompany the major components of the operating
    system on which the executable file runs.

 THIS SOFTWARE IS PROVIDED BY ORACLE CORPORATION ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 NON-INFRINGEMENT, ARE DISCLAIMED.  IN NO EVENT SHALL ORACLE CORPORATION
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
"""

import cmd, gettext, os, re, sys
import shlex
from ovmcli import *
import constants as CONSTANTS

_ = gettext.gettext

class OvmShell(cmd.Cmd):
    def __init__(self, ovm):
        cmd.Cmd.__init__(self)
        self.ovm = ovm
        self.prompt = self._getprompt()
        try:
            import readline
            delims = ''
            readline.set_completer_delims(delims)
        except ImportError:
            pass
        
    def default(self, line):
        COMMENT = re.compile("^\s*#")
        if COMMENT.match(line):
            return
        line = line.rstrip(";")
        words = shlex_split(line)
        if words is None:
            return
        if len(words) == 0:
            return
        if len(words) > 0 and words[0] == 'ovm':
            words = words[1:]
            if len(words) == 0:
                words.append('help')
        if len(words[0]) == 0:
            return       
        if words[0] in ['quit', 'exit', 'bye']:
            sys.exit(0)
        if words[0][0] == '@':
            self.do_script(line[1:])
        elif len(words) > 0:
            self.ovm.runcmd(words)
            self.prompt = self._getprompt()
            
    def emptyline(self):
        pass

    def get_names(self):
        result = []
        for n in self.ovm._get_complete_names():
            result.append(n)
        return result

    def complete(self, text, state):
        """
        Override cmd.Cmd.complete
        """
        n = len(text)
        HELP = 'help '
        completion_matches = []
        names = self.get_names()
        if text.startswith(HELP):
            names.append('all')
        for subcmd in names:
            if text.startswith(HELP):
                m = n - len(HELP)
                if subcmd[:m] == text.replace(HELP, ''):
                    completion_matches.append(HELP + subcmd)
            elif subcmd[:n] == text:
                completion_matches.append(subcmd)
        try:
            return completion_matches[state]
        except IndexError:
            return None
 
    def do_help(self, line):
        line = line.rstrip(";")
        words = shlex_split(line)
        if words is None:
            return
        self.ovm.do_help(words)

    def do_script(self, line):
        if not os.path.exists(line):
            print _('Script file "%(name)s" does not exist') %{"name":line}
            return
        cmds = ""
        COMMENT = re.compile("^\s*#")
        INIFILE = open(line, 'r')
        while 1:
            text = INIFILE.readline()
            if text in ['', None]:
                break
            if COMMENT.match(text):
                continue
            cmds += text
        cmds = cmds.split(";")    
        for cmd in cmds:
            cmd = cmd.strip()
            if cmd != '':
                words = shlex_split(cmd)
                if words is None:
                    continue
                if words[0] == "sleep":
                    if len(words) == 2:
                        seconds = words[1]
                    else:
                        seconds = "1"
                    self.do_sleep(seconds)
                else:
                    self.ovm.runcmd(words)
                
    def do_sleep(self, line):
        import time
        line = line.rstrip(";")
        seconds = 1
        try:
            if line == "":
                line = "1"
            seconds = int(line)     
        except Exception, e:
            print _("Error occurred while calling sleep. Please check your parameter")
            return       
        print _("Sleeping for %(number)d seconds") % {"number":seconds}
        time.sleep(seconds)
        
    def do_EOF(self, line):
        print "Bye."
        return True

    def _getprompt(self):
        default_prompt = "ovm> "
        serverpool_name = None
        if os.environ.has_key(CONSTANTS.SERVERPOOL_NAME):
            serverpool_name = os.environ[CONSTANTS.SERVERPOOL_NAME]
        if serverpool_name is not None and len(serverpool_name)!=0:
            prompt =  serverpool_name + "@" + default_prompt
        else:
            prompt = default_prompt
        if not sys.stdin.isatty():
            prompt += '\n'
        return prompt
         
def shlex_split(s, comments=False):

    lex = shlex.shlex(s, posix=True)
    lex.escape = ""
    lex.whitespace_split = True
    if not comments:
        lex.commenters = ''
    result = None
    try:
        result = list(lex)
    except ValueError, msg:
        print msg
    return result    