#!/usr/bin/python

"""
 Copyright (c) 2009
      Oracle Corporation.  All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Redistributions in any form must be accompanied by information on
    how to obtain complete source code for the Oracle VM CLI and any
    accompanying software that uses the Oracle VM CLI.  The source code
    must either be included in the distribution or be available for no
    more than the cost of distribution plus a nominal fee, and must be
    freely redistributable under reasonable conditions.  For an
    executable file, complete source code means the source code for all
    modules it contains.  It does not include source code for modules or
    files that typically accompany the major components of the operating
    system on which the executable file runs.

 THIS SOFTWARE IS PROVIDED BY ORACLE CORPORATION ``AS IS'' AND ANY EXPRESS
 OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 NON-INFRINGEMENT, ARE DISCLAIMED.  IN NO EVENT SHALL ORACLE CORPORATION
 BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
"""

import sys, os, optparse, re, types, gettext
import datetime
import md5
import ConfigParser
from ovmconfigfile import *
from OvmOption import *   
from ServerPoolManagementPort import *
from VirtualMachineManagementPort import *
from ResourceManagementPort import *
from LifecycleManagementPort import *
from AdminManagementPort import *

from ServerPoolManagementPort import ServerPoolManagementPort as spm
from VirtualMachineManagementPort import VirtualMachineManagementPort as vmm
from ResourceManagementPort import ResourceManagementPort as rsm
from LifecycleManagementPort import LifecycleManagementPort as lfm
from AdminManagementPort import AdminManagementPort as adm

from cmds import OvmCmd
from OvmShell import *
import constants as CONSTANTS
from errorcode import *
from util import getpass, uniq, indent, getusername
from ovmlogging import ovmlogging
from ZSI.auth import AUTH

_ = gettext.gettext

class OvmCli:
    def __init__(self, 
                 hostname = "", 
                 port = "", 
                 username = "", 
                 password = "", 
                 deploy_path = "", 
                 https = None, 
                 logmode = None, 
                 sep = ' '):
        
        self.OVM_VERSION = None
        self.datamodel = None          
        self.username = username
        self.password = password 
        self.session = None
        if https == 0:
            protocol = 'http'
            self.transport = None
        else:
            protocol = 'https'
            self.transport = None
        self.urlbase = "%s://%s:%s/%s" % (protocol, hostname, port, deploy_path)
        
        if username and password and len(username + password) != 0:
            self._update_object(username=username, password=password)
            
        self.shell = False
        self.sep = sep
        logclass = ovmlogging(mode = logmode)
        self.log = logclass.log
        self.cmd = OvmCmd()
        self.init_option_parser()

    def runcmd(self, args=None):
        if args is None:
            args = sys.argv[1:]
        if not args:
            return self.usage()

        (cmd, args) = self._get_cmd(args)
        if (cmd, args) == (None, None):
            return OVM_FAIL
     
        method_name = 'do_' + cmd
        try:
            method = getattr(self, method_name)
        except AttributeError:
            if self.shell:
                return self.usage(_("Subcommand \"%(cmd)s\" unknown. Type \"help\" for a list of subcommands.") 
                                  % {"cmd":cmd})
            else:
                print _("Subcommand \"%(cmd)s\" unknown.") % {"cmd":cmd}
                return OVM_FAIL

        if cmd in ['help', 'config']:
            return method(args)
        
        if cmd in CONSTANTS.SHELL_ONLY and '--help' not in args and'-h' not in args:
            if not self.shell:
                print "Subcommand \"%(name)s\" is only available for shell sessions." %{"name":cmd}
                return OVM_FAIL
                    
        if cmd not in CONSTANTS.NOLOG_CMDS and '--help' not in args and'-h' not in args:
            if not self._check_session():
                return OVM_FAIL
        
        if cmd in CONSTANTS.GROUP_CMDS and ('--help' in args or '-h' in args) :
            self._print_group_help(cmd)
            return

        if cmd in self.cmd.get_all_subgroup_cmds() and ('--help' in args or '-h' in args):
            self._print_subgroup_help(cmd)
            return
 
        parser = self.get_parser(cmd)
        options, args = parser.parse_args(args)            
        if parser.is_error():
            print parser.get_error_msg()
            return OVM_OPTION_ERROR
        return method(options)
            
    
    def usage(self, msg=None):
        if msg is None:
            usage = _("Usage: ovm [options] subcommand [suboptions]")
            if self.shell:
                usage = _("Usage: subcommand [suboptions]")
                
            print usage + _("""
\nOracle VM Manager Command Line Interface\n
Subcommands:\n""")
   
            group_infos = self.cmd.get_group_infos()
            groups = group_infos.keys()
            groups.sort()
            for info in groups:
                print (CONSTANTS.HELP_MSG_FORMAT % (info, group_infos[info]))                  
            self._print_misc_help()
            self._print_foot_help()
        else:
            print msg
      
    def init_option_parser(self):
        self.parser = {}
        SERVERPOOL_NAME = _('Server pool name')
        VM_NAME         = _("Virtual machine name")
        SERVER_NAME     = _("Server name")
        TEMPLATE_NAME   = _("Virtual machine template name")
        VM_IMAGE_NAME   = _("Virtual machine image name")
        ISO_IMAGE_GROUP = _("ISO image group")
        ISO_IMAGE_LABEL = _("ISO image label")

        shell = self.get_parser('shell', '[options]')
        shell.add_option('--script', '-s', help = _("Location of script file"))
        shell.add_option("-L", "--nolog", action="store_true", \
                           default=False, help=_("Begin shell session without connecting to the server"))        
        config = self.get_parser('config', '[options]')
        config.add_option('--hostname', '-H', help = _("Oracle VM Manager hostname"))
        config.add_option('--port', '-p', help = _("Oracle VM Manager port number"))
        config.add_option('--deploy_path', '-D', help = _("Deploy path"))
        config.add_option('--vncviewer', '-V', help = _("Full path to vncviewer"))
        config.add_option('--enable_https', '-e', action="store_true", \
                           default=False, help = _("Enable HTTPS support"))
        config.add_option('--disable_https', '-d', action="store_true", \
                           default=False, help = _("Disable HTTPS support"))         

        help   = self.get_parser('help', '[options] [subcommand]')

        val_os = self.get_parser('val_os', '[options]')
        val_kb = self.get_parser('val_kb', '[options]')
        val_bootdev = self.get_parser('val_bootdev', '[options]')

        home = self.get_parser('home', '[options]')

        connect = self.get_parser('connect', '[options]')
        connect.add_option('--username', '-u', help = _("Oracle VM Manager username"))
        connect.add_option('--password', '-p', help = _("Oracle VM Manager user password"))
                        
        vm_ls = self.get_parser('vm_ls', '[options]')
        vm_ls.add_option('--vm_name', '-n', help = VM_NAME)
        vm_ls.add_option('--serverpool_name', '-s', help = SERVERPOOL_NAME)
        vm_ls.add_option('--show_server', '-l',  action="store_true", \
                           default=False, help = _("Display the server where the virtual machine is running"))
        
        vm_new = self.get_parser('vm_new', '[options]')
        vm_new.add_option('--method', '-m', required=True, \
                     help=_("Virtual machine creation method (template/iso/pxe)"))
        
        vm_new.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        #vm_new.add_option('--source_serverpool_name', '-T', help=_("Source server pool name"))   
        vm_new.add_option('--template', '-t', help= TEMPLATE_NAME)
        vm_new.add_option('--vm_name', '-n', required=True, help=VM_NAME)
        vm_new.add_option('--console_password', '-c', help=_("Console password"))
        vm_new.add_option('--ha_enabled', '-a', action="store_true", \
                           default=False, help=_("Enable high availability"))
        vm_new.add_option('--hvm', '-x', action="store_true", \
                           default=False, help=_("Create a hardware virtualized guest"))
        vm_new.add_option('--iso_group', '-g', help= ISO_IMAGE_GROUP)
        vm_new.add_option('--iso_label', '-l', help= ISO_IMAGE_LABEL)
        vm_new.add_option('--location', '-L', help=_("Paravirtualized guest install source URL \
                                                        (e.g., http://host/path, ftp://host/path, nfs:host:/path)"))
        vm_new.add_option('--os', '-o',  help=_("Operating system type"))
        vm_new.add_option('--vcpus', '-V', type = "int", default=1, help=_("Number of VCPUs"))
        vm_new.add_option('--memory', '-y', type = "int", help=_("Memory size in megabytes"))
        vm_new.add_option('--disksize', '-d', type = "float", help=_("Virtual disk size in megabytes"))
        vm_new.add_option('--preferred_server', '-p', help=_("Preferred servers (comma separated)"))
        
        vm_stat = self.get_parser('vm_stat', '[options]')
        vm_stat.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_stat.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_stat.add_option('--reset', '-r',  action="store_true", \
                           default=False, help=_("Reset virtual machine status to Error"))
        
        vm_info = self.get_parser('vm_info', '[options]')
        vm_info.add_option('--serverpool_name', '-s', required=True, check =False, help = SERVERPOOL_NAME)
        vm_info.add_option('--vm_name', '-n', required=True, help = VM_NAME)
    
        vm_del = self.get_parser('vm_del', '[options]')
        vm_del.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_del.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_del.add_option('--safe', '-f',  action="store_true", \
                           default=False, help = _("Delete the virtual machine without removing files"))        
 
        vm_save = self.get_parser('vm_save', '[options]')
        vm_save.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_save.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_save.add_option('--template_name', '-t', required=True, help = TEMPLATE_NAME)               
        
        vm_poweron = self.get_parser('vm_poweron', '[options]')
        vm_poweron.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_poweron.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_poweron.add_option('--force', '-f',  action="store_true", \
                           default=False, help = _("Force virtual machine power on"))
        
        vm_reboot = self.get_parser('vm_reboot', '[options]')
        vm_reboot.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_reboot.add_option('--vm_name', '-n', required=True, help = VM_NAME)

        vm_pause = self.get_parser('vm_pause', '[options]')
        vm_pause.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_pause.add_option('--vm_name', '-n', required=True, help = VM_NAME)

        vm_unpause = self.get_parser('vm_unpause', '[options]')
        vm_unpause.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_unpause.add_option('--vm_name', '-n', required=True, help = VM_NAME)

        vm_suspend = self.get_parser('vm_suspend', '[options]')
        vm_suspend.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_suspend.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        
        vm_resume = self.get_parser('vm_resume', '[options]')
        vm_resume.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_resume.add_option('--vm_name', '-n', required=True, help = VM_NAME)

        vm_poweroff = self.get_parser('vm_poweroff', '[options]')
        vm_poweroff.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_poweroff.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_poweroff.add_option('--force', '-f',  action="store_true", \
                           default=False, help = _("Force virtual machine power off"))
        
        vm_conf = self.get_parser('vm_conf', '[options]')
        vm_conf.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_conf.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_conf.add_option('--new_name', '-N', help = _("New virtual machine name"))
        vm_conf.add_option('--new_maxmem', '-x', type="int", help = _("New max memory size in megabytes"))
        vm_conf.add_option('--new_memory', '-m', type="int", help = _("New memory size in megabytes"))
        vm_conf.add_option('--new_vcpus', '-c', type="int", help = _("New number of VCPUs"))

        vm_conf.add_option('--new_priority', '-r', type="int", help = _("New VCPU scheduling priority (1-100) \t\t\t\t100=High, 50=Intermediate, 1=Low"))
        vm_conf.add_option('--new_cap', '-C', type="int", help = _("New VCPU scheduling cap (10-100) \t\t\t\t100=High, 50=Intermediate, 10=Low"))
        vm_conf.add_option('--new_group', '-g', help = _("New group"))

        vm_conf.add_option('--new_os', '-o', help = _("New operating system type"))
        vm_conf.add_option('--network_type', '-t', help = _("New network type (ioemu/netfront)"))
        vm_conf.add_option('--preferred_servers', '-p', help = _("Preferred servers (comma separated)"))  
        vm_conf.add_option('--preferred_servers_auto', '-P',  action="store_true", \
                           default=False, help = _("Set preferred server mode to AUTO"))
        vm_conf.add_option('--enable_ha', '-e',  action="store_true", \
                           default=False, help = _("Enable high availability"))
        vm_conf.add_option('--disable_ha', '-d',  action="store_true", \
                           default=False, help = _("Disable high availability"))
        vm_conf.add_option('--new_description', '-D', help = _("New description"))
        
        vm_disk_add = self.get_parser('vm_disk_add', '[options]')
        vm_disk_add.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_disk_add.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_disk_add.add_option('--disk_name', '-d', required=True, help = _("Virtual disk name"))
        vm_disk_add.add_option('--disk_size', '-S', type = "long", required=True, help = _("Virtual disk size in megabytes"))
        vm_disk_add.add_option('--drive_type', '-t', default="auto", help = _("Hard drive type(default: auto)"))
        vm_disk_add.add_option('--priority_class', '-p', type = "int", default=0, \
                           help = _("Priority class (0-7) - requires QoS"))
        vm_disk_add.add_option('--enable_qos', '-q',  action="store_true", \
                           default=False, help = _("Enable QoS"))

        vm_disk_conf = self.get_parser('vm_disk_conf', '[options]')
        vm_disk_conf.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_disk_conf.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_disk_conf.add_option('--disk_name', '-d', required=True, help = _("Virtual disk name"))
        vm_disk_conf.add_option('--drive_type', '-t', help = _("Hard drive type (auto/hd/sd/xvd)"))
        vm_disk_conf.add_option('--priority_class', '-p', type = "int", \
                           help = _("Priority class (0-7) - requires QoS"))
        vm_disk_conf.add_option('--enable_qos', '-q',  action="store_true", \
                           default=False, help = _("Enable QoS"))
        vm_disk_conf.add_option('--disable_qos', '-D',  action="store_true", \
                           default=False, help = _("Disable QoS"))
                
        vm_disk_del = self.get_parser('vm_disk_del', '[options]')
        vm_disk_del.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_disk_del.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_disk_del.add_option('--disk_name', '-d', required=True, help = _("Virtual disk name"))
                
        vm_disk_ls = self.get_parser('vm_disk_ls', '[options]')
        vm_disk_ls.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_disk_ls.add_option('--vm_name', '-n', required=True, help = VM_NAME)
            
        vm_sd_attach = self.get_parser('vm_sd_attach', '[options]')
        vm_sd_attach.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_sd_attach.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_sd_attach.add_option('--disk_name', '-d', required=True, help = _("Shared virtual disk name"))
    

        vm_sd_detach = self.get_parser('vm_sd_detach', '[options]')
        vm_sd_detach.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_sd_detach.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_sd_detach.add_option('--disk_name', '-d', required=True, help = _("Shared virtual disk name"))
        
        vm_nic_del = self.get_parser('vm_nic_del', '[options]')
        vm_nic_del.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_nic_del.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_nic_del.add_option('--nic_name', '-N', required=True, help = _("NIC name"))
        
        vm_nic_add = self.get_parser('vm_nic_add', '[options]')
        vm_nic_add.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_nic_add.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_nic_add.add_option('--nic_name', '-N', required=True, help = _("NIC name"))
        vm_nic_add.add_option('--bridge', '-b', required=True, check=False, help = _("Network bridge"))
        vm_nic_add.add_option('--rate_limit', '-r', type = "float", default=0, help = _("Rate limit in megabits"))
        vm_nic_add.add_option('--enable_qos', '-e',  action="store_true", \
                           default=False, help = _("Enable rate limit"))

        vm_nic_conf = self.get_parser('vm_nic_conf', '[options]')
        vm_nic_conf.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_nic_conf.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_nic_conf.add_option('--nic_name', '-N', required=True, help = _("NIC name"))
        vm_nic_conf.add_option('--new_nic_name', '-i', help = _("New NIC name"))
        vm_nic_conf.add_option('--bridge', '-b', help = _("Network bridge"))
        vm_nic_conf.add_option('--rate_limit', '-r', type = "float", default=0, help = _("Rate limit in megabits"))
        vm_nic_conf.add_option('--enable_qos', '-e',  action="store_true", \
                           default=False, help = _("Enable rate limit"))
        vm_nic_conf.add_option('--disable_qos', '-d',  action="store_true", \
                           default=False, help = _("Disable rate limit"))
                        
        vm_nic_ls = self.get_parser('vm_nic_ls', '[options]')
        vm_nic_ls.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_nic_ls.add_option('--vm_name', '-n', required=True, help = VM_NAME)

        vm_cd_attach = self.get_parser('vm_cd_attach', '[options]')
        vm_cd_attach.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_cd_attach.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_cd_attach.add_option('--iso_group', '-g', required=True, help = ISO_IMAGE_GROUP)
        vm_cd_attach.add_option('--iso_label', '-l', required=True, help = ISO_IMAGE_LABEL)

        vm_cd_detach = self.get_parser('vm_cd_detach', '[options]')
        vm_cd_detach.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_cd_detach.add_option('--vm_name', '-n', required=True, help = VM_NAME)

        vm_cd_ls = self.get_parser('vm_cd_ls', '[options]')
        vm_cd_ls.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_cd_ls.add_option('--vm_name', '-n', required=True, help = VM_NAME)

        vm_bootdev = self.get_parser('vm_bootdev', '[options]')
        vm_bootdev.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_bootdev.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_bootdev.add_option('--boot_device', '-b', required=True, 
                                     help = _("Boot device types: %s, %s, %s" % 
                                              (CONSTANTS.BOOT_DEVICE_CDROM, CONSTANTS.BOOT_DEVICE_HDD, CONSTANTS.BOOT_DEVICE_PXE )))

        vm_vncpwd = self.get_parser('vm_vncpwd', '[options]')
        vm_vncpwd.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_vncpwd.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_vncpwd.add_option('--console_password', '-c', help = _("VNC console password"))              

        vm_kb = self.get_parser('vm_kb', '[options]')
        vm_kb.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_kb.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_kb.add_option('--keyboard', '-k', required=True, check=False, help = _("Keyboard layout"))
                                                        
        vm_clone = self.get_parser('vm_clone', '[options]')
        vm_clone.add_option('--serverpool_name', '-s', required=True, check=False, help = _("Source server pool name"))
        vm_clone.add_option('--source_vm_name', '-n', required=True, help = _("Source virtual machine name"))
        vm_clone.add_option('--target_vm_prefix', '-p', required=True, help = _("Target virtual machine name prefix"))
        vm_clone.add_option('--target_serverpool_name', '-t',  help = _("Target server pool"))
        vm_clone.add_option('--group', '-g', default=CONSTANTS.MYWORKSPACE, 
                                  help = _("Group name(default:%s)"% CONSTANTS.MYWORKSPACE))        
        vm_clone.add_option('--num_of_copy', '-c', type="int", default=1, help = _("Number of copies"))

        vm_deploy = self.get_parser('vm_deploy', '[options]')
        vm_deploy.add_option('--serverpool_name', '-s', required=True, check=False, help = _("Source server pool name"))
        vm_deploy.add_option('--source_vm_name', '-n', required=True, help = _("Source virtual machine name"))
        vm_deploy.add_option('--target_vm_name', '-N', required=True, help = _("Target virtual machine name"))
        vm_deploy.add_option('--target_serverpool_name', '-t',  help = _("Target server pool"))
        vm_deploy.add_option('--target_group', '-g', required=True, help = _("Target group"))
                
        vm_mig = self.get_parser('vm_mig', '[options]')
        vm_mig.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_mig.add_option('--vm_name', '-n', required=True, help = VM_NAME)
        vm_mig.add_option('--target_server', '-t', help = _("Target server"))

        vm_mig_all = self.get_parser('vm_mig_all', '[options]')
        vm_mig_all.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vm_mig_all.add_option('--server_name', '-n', required=True, help = _("Source server"))
        vm_mig_all.add_option('--target_server', '-t', help = _("Target server"))

        tmpl_imp = self.get_parser('tmpl_imp', '[options]')
        tmpl_imp.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        tmpl_imp.add_option('--template_name', '-n', required=True, help = TEMPLATE_NAME)
        tmpl_imp.add_option('--username', '-u', required=True,  help = _("Virtual machine system username"))
        tmpl_imp.add_option('--password', '-p', help = _("Virtual machine system password"))
        tmpl_imp.add_option('--url', '-l', required=True, help = _("Source location URL (e.g., http://host/vm)"))
        tmpl_imp.add_option('--proxy', '-X', help = _("Proxy URL"))
        tmpl_imp.add_option('--os_type', '-o', required=True, help = _("Operating system type"))        
        tmpl_imp.add_option('--description', '-d', help = _("Description"))
                    
        tmpl_reg = self.get_parser('tmpl_reg', '[options]')
        tmpl_reg.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        tmpl_reg.add_option('--template_name', '-n', required=True, help = TEMPLATE_NAME)
        tmpl_reg.add_option('--username', '-u', required=True,  help = _("Virtual machine system username"))
        tmpl_reg.add_option('--password', '-p', help = _("Virtual machine system password"))
        tmpl_reg.add_option('--os_type', '-o', required=True, help = _("Operating system type"))       
        tmpl_reg.add_option('--description', '-d', help = _("Description"))

        tmpl_disc = self.get_parser('tmpl_disc', '[options]')
        tmpl_disc.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
                
        tmpl_ls = self.get_parser('tmpl_ls', '[options]')
        tmpl_ls.add_option('--serverpool_name', '-s',  help = SERVERPOOL_NAME)
        
        tmpl_del = self.get_parser('tmpl_del', '[options]')
        tmpl_del.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        tmpl_del.add_option('--template_name', '-n', required=True, help = TEMPLATE_NAME)           

        tmpl_stat = self.get_parser('tmpl_stat', '[options]')
        tmpl_stat.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        tmpl_stat.add_option('--template_name', '-n', required=True, help = TEMPLATE_NAME) 
        tmpl_stat.add_option('--progress', '-p', action="store_true", \
                           default=False, help = _("Display import progress")) 
         
        tmpl_approve = self.get_parser('tmpl_approve', '[options]')
        tmpl_approve.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        tmpl_approve.add_option('--template_name', '-n', required=True, help = TEMPLATE_NAME)
                
        svrp_new = self.get_parser('svrp_new', '[options]')
        svrp_new.add_option('--hostname', '-H', required=True, help = _("Server hostname or IP address"))
        svrp_new.add_option('--serverpool_name', '-s', required=True, help = SERVERPOOL_NAME)
        svrp_new.add_option('--vip', '-i', help = _("Server pool virtual IP"))
        svrp_new.add_option('--ha_enabled', '-a', action="store_true", 
                                     default=False, help = _("Enable high availability"))
        svrp_new.add_option('--agent_password', '-A', help = _("Oracle VM Agent password"))
        #svrp_new.add_option('--server_type', '-t', help = "Server Type")
        svrp_new.add_option('--utility_username', '-U', required=True, help = _("Utility server username"))
        svrp_new.add_option('--utility_password', '-P',  help = _("Utility server password"))
        svrp_new.add_option('--user_list', '-u',  help = _("Non-admin username list (comma separated)"))
        svrp_new.add_option('--server_name', '-n',  help = _("Server name"))
        svrp_new.add_option('--server_location', '-L', help = _("Server location"))
        svrp_new.add_option('--description', '-D', help = _("Description"))
        
        svrp_del = self.get_parser('svrp_del', '[options]')
        svrp_del.add_option('--serverpool_name', '-s', required=True, help = SERVERPOOL_NAME)
        svrp_del.add_option('--isdelworkdirectory', '-d', 
                                     action="store_true",  default = False,
                                     help = _("Remove all working directories on the server pool"))
        svrp_del.add_option('--force', '-f', 
                                     action="store_true",  default = False,
                                     help = _("Force server pool delete"))
        
        svrp_stat = self.get_parser('svrp_stat', '[options]')
        svrp_stat.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
    
        svrp_info = self.get_parser('svrp_info', '[options]')
        svrp_info.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)

        svrp_conf = self.get_parser('svrp_conf', '[options]')
        svrp_conf.add_option('--serverpool_name', '-s', required=True, help = SERVERPOOL_NAME)        
        svrp_conf.add_option('--enable_ha', '-e', action="store_true", 
                                     default=False, help = _("Enable high availability"))
        svrp_conf.add_option('--disable_ha', '-d', action="store_true", 
                                     default=False, help = _("Disable high availability"))
        svrp_conf.add_option('--new_poolname', '-n', help = _("New server pool name"))
        svrp_conf.add_option('--vip', '-i', help = _("Server pool virtual IP"))   
                   
        svrp_restore = self.get_parser('svrp_restore', '[options]')
        svrp_restore.add_option('--serverpool_name', '-s', required=True, help = SERVERPOOL_NAME)

        svrp_refresh = self.get_parser('svrp_refresh', '[options]')
        svrp_ls = self.get_parser('svrp_ls', '[options]')
        
        iso_reg = self.get_parser('iso_reg', '[options]')
        iso_reg.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        iso_reg.add_option('--iso_group', '-g', required=True, help = ISO_IMAGE_GROUP)
        iso_reg.add_option('--iso_label', '-l', required=True,  help = ISO_IMAGE_LABEL)      
        iso_reg.add_option('--description', '-d', help = _("Description"))

        iso_disc = self.get_parser('iso_disc', '[options]')
        iso_disc.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
                
        iso_del = self.get_parser('iso_del', '[options]')
        iso_del.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        iso_del.add_option('--iso_group', '-g', required=True, help = ISO_IMAGE_GROUP)
        iso_del.add_option('--iso_label', '-l', required=True, help = ISO_IMAGE_LABEL)

        iso_stat = self.get_parser('iso_stat', '[options]')
        iso_stat.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        iso_stat.add_option('--iso_group', '-g', required=True, help = ISO_IMAGE_GROUP)
        iso_stat.add_option('--iso_label', '-l', required=True, help = ISO_IMAGE_LABEL)
        iso_stat.add_option('--progress', '-p', action="store_true", \
                           default=False, help = _("Display import progress"))
                                           
        iso_approve = self.get_parser('iso_approve', '[options]')
        iso_approve.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        iso_approve.add_option('--iso_group', '-g', required=True, help = ISO_IMAGE_GROUP)
        iso_approve.add_option('--iso_label', '-l', required=True, help = ISO_IMAGE_LABEL)
        
        iso_ls = self.get_parser('iso_ls', '[options]')
        iso_ls.add_option('--serverpool_name', '-s',  help = SERVERPOOL_NAME)

        iso_imp = self.get_parser('iso_imp', '[options]')
        iso_imp.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        iso_imp.add_option('--iso_group', '-g', required=True, help = ISO_IMAGE_GROUP)
        iso_imp.add_option('--iso_label', '-l', required=True,  help = ISO_IMAGE_LABEL)
        iso_imp.add_option('--url', '-u', required=True, help = _("Resource source URL"))
        iso_imp.add_option('--proxy', '-X', help = _("Proxy URL"))     
        iso_imp.add_option('--description', '-d', help = _("Description"))

        img_imp = self.get_parser('img_imp', '[options]')
        img_imp.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        img_imp.add_option('--image_name', '-n', required=True, help = VM_IMAGE_NAME)
        img_imp.add_option('--enable_ha', '-H', action="store_true", 
                                     default=False,  help = _("Enable high availability"))
        img_imp.add_option('--console_password', '-c',  help = _("VNC console password"))
        img_imp.add_option('--username', '-u',  help = _("Virtual machine system username"))
        img_imp.add_option('--password', '-p',  help = _("Virtual machine system password"))
        img_imp.add_option('--url', '-l', required=True, help = _("Resource source URL"))
        img_imp.add_option('--proxy', '-X', help = _("Proxy URL"))
        img_imp.add_option('--os_type', '-o', required=True, help = _("Operating system type"))
        img_imp.add_option('--group', '-g', default=CONSTANTS.MYWORKSPACE, 
                                  help = _("Group name(default:%s)"% CONSTANTS.MYWORKSPACE))         
        img_imp.add_option('--description', '-d', help = _("Description"))
        
        img_disc = self.get_parser('img_disc', '[options]')
        img_disc.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        
        img_reg = self.get_parser('img_reg', '[options]')
        img_reg.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        img_reg.add_option('--image_name', '-n', required=True, help = VM_IMAGE_NAME)
        img_reg.add_option('--username', '-u',  help = _("Virtual machine system username"))
        img_reg.add_option('--password', '-p', help = _("Virtual machine system password"))
        img_reg.add_option('--console_password', '-c', help = _("VNC console password"))
        img_reg.add_option('--os_type', '-o', required=True, help = _("Operating system type"))
        img_reg.add_option('--enable_ha', '-a', action="store_true", 
                                     default=False, help = _("Enable high availability")) 
        img_reg.add_option('--group', '-g', default=CONSTANTS.MYWORKSPACE, 
                                  help = _("Group name(default:%s)"% CONSTANTS.MYWORKSPACE))       
        img_reg.add_option('--description', '-d', help = _("Description"))

        img_approve = self.get_parser('img_approve', '[options]')
        img_approve.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        img_approve.add_option('--image_name', '-n', required=True, help = VM_IMAGE_NAME)
        
        img_del = self.get_parser('img_del', '[options]')
        img_del.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        img_del.add_option('--image_name', '-n', required=True, help = VM_IMAGE_NAME) 

        img_stat = self.get_parser('img_stat', '[options]')
        img_stat.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        img_stat.add_option('--image_name', '-n', required=True, help = VM_IMAGE_NAME) 
        img_stat.add_option('--progress', '-p', action="store_true", \
                           default=False, help = _("Display import progress"))
                           
        img_ls = self.get_parser('img_ls', '[options]')
        img_ls.add_option('--serverpool_name', '-s',  help = SERVERPOOL_NAME)
                
        svr_restart = self.get_parser('svr_restart', '[options]')
        svr_restart.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        svr_restart.add_option('--server_name', '-n', required=True, help = SERVER_NAME)

        svr_poweroff = self.get_parser('svr_poweroff', '[options]')
        svr_poweroff.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        svr_poweroff.add_option('--server_name', '-n', required=True, help = SERVER_NAME)


        svr_mode = self.get_parser('svr_mode', '[options]')
        svr_mode.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        svr_mode.add_option('--server_name', '-n', required=True, help = SERVER_NAME)
        svr_mode.add_option('--maintain', '-m', action="store_true", \
                           default=False, help = _("Put server in maintenance mode"))
        svr_mode.add_option('--normal', '-N', action="store_true", \
                           default=False, help = _("Put server in normal mode"))
        
        svr_info = self.get_parser('svr_info', '[options]')
        svr_info.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        svr_info.add_option('--server_name', '-n', required=True, help = SERVER_NAME)

        svr_stat = self.get_parser('svr_stat', '[options]')
        svr_stat.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        svr_stat.add_option('--server_name', '-n', required=True, help = SERVER_NAME)


        svr_del = self.get_parser('svr_del', '[options]')
        svr_del.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        svr_del.add_option('--server_name', '-n', required=True, help = SERVER_NAME)
        svr_del.add_option('--force', '-f', 
                                     action="store_true",  default = False,
                                     help = _("Force server delete"))                
        svr_ls = self.get_parser('svr_ls', '[options]')
        svr_ls.add_option('--serverpool_name', '-s',  help = SERVERPOOL_NAME)

        svr_conf = self.get_parser('svr_conf', '[options]')
        svr_conf.add_option('--serverpool_name', '-s',  required=True, check=False, help = SERVERPOOL_NAME)
        svr_conf.add_option('--server_name', '-n',  required=True, help = SERVER_NAME)
        svr_conf.add_option('--new_name', '-N',  help = _("New server name"))
        svr_conf.add_option('--agent_password', '-A',  help = _("Oracle VM Agent password"))
        svr_conf.add_option('--username', '-u',  help = _("Utility server username"))
        svr_conf.add_option('--password', '-p',  help = _("Utility server password"))
        svr_conf.add_option('--server_type', '-t',  \
                                 help = _("New server type (comma separated)\t\t\t\t m: Server Pool Master\t\t\t\t\t v: Virtual Machine Server\t\t\t\t\t u: Utility Server" ))
        svr_conf.add_option('--location', '-l',  help = _("New server location"))
        svr_conf.add_option('--description', '-d',  help = _("New description"))
        svr_conf.add_option('--force', '-f', action="store_true", \
                                 default=False, help = _("Force server update"))

        svr_pwd = self.get_parser('svr_pwd', '[options]')
        svr_pwd.add_option('--serverpool_name', '-s',  required=True, check=False, help = SERVERPOOL_NAME)
        svr_pwd.add_option('--server_name', '-n',  required=True, help = SERVER_NAME)
        svr_pwd.add_option('--agent', '-a', action="store_true", \
                                 default=False, help = _("Change agent password"))
        svr_pwd.add_option('--utility', '-u', action="store_true", \
                                 default=False, help = _("Change utility server password"))
        svr_pwd.add_option('--old_pwd', '-O',  help = _("Old password"))
        svr_pwd.add_option('--new_pwd', '-N',  help = _("New password"))

                                                
        svr_add = self.get_parser('svr_add', '[options]')
        svr_add.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        svr_add.add_option('--hostname', '-H', required=True, help = _("Server hostname or IP address"))
        svr_add.add_option('--server_name', '-n',  help = _("Server name"))
        svr_add.add_option('--agent_password', '-A',  help = _("Oracle VM Agent password"))
        svr_add.add_option('--utility_username', '-U', help = _("Utility server username"))
        svr_add.add_option('--utility_password', '-P', help = _("Utility server password"))
        svr_add.add_option('--vm_only', '-o', action="store_true", 
                                     default=False,help = _("Add server as a Virtual Machine Server only"))
        svr_add.add_option('--utility_only', '-u', action="store_true", 
                                     default=False,help = _("Add server as a Utility Server only"))
        svr_add.add_option('--server_location', '-L', help = _("Server location"))
        svr_add.add_option('--description', '-D', help = _("Description"))
                
        sd_new = self.get_parser('sd_new', '[options]')
        sd_new.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        sd_new.add_option('--disk_name', '-n', required=True, help = _("Shared disk name"))
        sd_new.add_option('--disk_size', '-d', type = "long", required=True, help = _("Shared disk size in megabytes"))
        sd_new.add_option('--group', '-g', default=CONSTANTS.MYWORKSPACE, 
                                  help = _("Group name(default:%s)"% CONSTANTS.MYWORKSPACE)) 
        sd_new.add_option('--comments', '-c', help = _("Comments"))

        sd_del = self.get_parser('sd_del', '[options]')
        sd_del.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        sd_del.add_option('--disk_name', '-n', required=True, help = _("Shared disk name")) 

        sd_stat = self.get_parser('sd_stat', '[options]')
        sd_stat.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        sd_stat.add_option('--disk_name', '-n', required=True, help = _("Shared disk name"))

        sd_ls = self.get_parser('sd_ls', '[options]')
        sd_ls.add_option('--serverpool_name', '-s',  help = SERVERPOOL_NAME)
        

        sd_disc = self.get_parser('sd_disc', '[options]')
        sd_disc.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        

        sd_reg = self.get_parser('sd_reg', '[options]')
        sd_reg.add_option('--serverpool_name', '-s',  help = SERVERPOOL_NAME)
        sd_reg.add_option('--group', '-g', default=CONSTANTS.MYWORKSPACE, 
                                  help = _("Group name(default:%s)"% CONSTANTS.MYWORKSPACE)) 
        sd_reg.add_option('--multipath', '-m', action="store_true", 
                                     default=False,help = _("Register a multipath-based disk"))
        sd_reg.add_option('--disk_name', '-n', required=True, help = _("Shared disk name"))
        sd_reg.add_option('--comments', '-c', help = _("Comments"))
                                 
        svr_agent = self.get_parser('svr_agent', '[options]')
        svr_agent.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        svr_agent.add_option('--server_name', '-n', required=True, help = SERVER_NAME)
        
        user_ls = self.get_parser('user_ls', '[options]')
        
        user_new = self.get_parser('user_new', '[options]')
        user_new.add_option('--username', '-u', required=True, help = _("Username"))
        user_new.add_option('--password', '-p', help = _("Password"))
        user_new.add_option('--email', '-e', required=True, help = _("Email address"))
        user_new.add_option('--firstname', '-f',  help = _("First name"))
        user_new.add_option('--lastname', '-l',  help = _("Last name"))
        user_new.add_option('--role_name', '-r', required=True, check=False, help = _("Role name"))
        user_new.add_option('--disable', '-d', action="store_true", 
                                     default=False,help = _("Set user status to LOCKED"))
        
        group_ls = self.get_parser('group_ls', '[options]')
        
        group_new = self.get_parser('group_new', '[options]')
        group_new.add_option('--groupname', '-n', required=True, help = _("Group name"))
        group_new.add_option('--desc', '-d', help = _("Description"))

        user_group_add = self.get_parser('user_group_add', '[options]')
        user_group_add.add_option('--username', '-u', required=True, help = _("Username"))
        user_group_add.add_option('--groupname', '-g', required=True, help = _("Group"))

        
        user_svrp_add = self.get_parser('user_svrp_add', '[options]')
        user_svrp_add.add_option('--username', '-u', required=True, help = _("Username"))
        user_svrp_add.add_option('--serverpool_name', '-s', required=True, help = SERVERPOOL_NAME)

        user_svrp_del = self.get_parser('user_svrp_del', '[options]')
        user_svrp_del.add_option('--username', '-u', required=True, help = _("Username"))
        user_svrp_del.add_option('--serverpool_name', '-s', required=True, help = SERVERPOOL_NAME)
               
        vncviewer = self.get_parser('vncviewer', '[options]')
        vncviewer.add_option('--serverpool_name', '-s', required=True, check=False, help = SERVERPOOL_NAME)
        vncviewer.add_option('--vm_name', '-n', required=True, help = VM_NAME)

        use = self.get_parser('use', '[options]')
        use.add_option('--serverpool_name', '-s', required=True, help = SERVERPOOL_NAME)
           
    def get_parser(self, prog, usage=None):
        if not self.parser.has_key(prog):
            desc = self.cmd.get_cmd_desc(prog)
            #desc = 'Description: ' + desc
            if usage is not None:
                if self.shell:
                    usage = _('%(prog)s %(usage)s\n%(desc)s') % {"prog":prog, "usage":usage, "desc":desc}
                else:
                    usage = _('ovm %(prog)s %(usage)s\n%(desc)s') % {"prog":prog, "usage":usage, "desc":desc}
            self.parser[prog] = OvmOptionParser(prog=prog, usage=usage,
                                             option_class=OvmOption)
        return self.parser[prog]          
           
    def do_shell(self, options):
        """ shell --- Launch an interactive shell"""      

        script = options.script
        nolog  = options.nolog

        if not nolog:
            if self.username is None:
               self.username = getusername()
               if self.username == CONSTANTS.EOF:
                   return 
            if self.password is None:
                self.password = getpass(_("Password: "))

            self._update_object(username=self.username, password = self.password)
                               
        shell = OvmShell(self)
        self.shell = True

        if nolog and self.shell:
            self.session = None
            
        self.init_option_parser()
        if script is None:
            if not nolog:
                sp = self.spm.getAllServerPools()
                if sp is None:
                    return OVM_FAIL
            shell.cmdloop(_('Type "help" for a list of commands.'))
        else:
            shell.do_script(script)
        
    def do_help(self, argv):
        """ help --- Show help"""
        parser = self.get_parser('help')
        options, args = parser.parse_args(argv)
        
        if parser.is_error():
            print parser.get_error_msg()
            return OVM_OPTION_ERROR
        if len(argv):
            if type(argv) == type([]):
                group = argv[0]
                if group == 'all':
                   self._print_group_help(group)
                   return                      
                else:
                    if self.cmd.is_group(group):
                        strcmd = ' '.join(argv[1:])

                        if len(strcmd) == 0:
                            if self.cmd.get_subcmds_by_group(group):
                                self._print_group_help(group)
                                return
                        else:
                            disp = group +" " + strcmd
                            subgroup_cmd = group + '_' + strcmd
                            if subgroup_cmd in self.cmd.get_all_subgroup_cmds():
                                self._print_subgroup_help(subgroup_cmd)
                                return
                            
                            if not self.cmd.get_subcmd_by_group(group, strcmd):
                                print _('Help not available for \"%(cmd)s\"' % {"cmd":disp})
                                return
                    else:
                         print _('Help not available for \"%(cmd)s\"' % {"cmd":" ".join(argv)})
                         return
                     
                if group is None:
                    cmd = strcmd
                else:                                
                    cmd = group
                    if strcmd:       
                        cmd += "_" + strcmd                
                cmd = cmd.replace(' ', '_')
            else:
                cmd = argv
   
            if cmd is None:
                print _('Help not available for \"%(cmd)s\"' % {"cmd":cmd})
                return OVM_OPTION_ERROR
            print self.get_parser(cmd).format_help()
        else:
            return self.usage()
        
    def do_config(self, argv):
        """ config --- Start a configuration wizard"""
        parser = self.get_parser('config')
        options, args = parser.parse_args(argv)
        
        if parser.is_error():
            print parser.get_error_msg()
            return OVM_OPTION_ERROR
        
        hostname = options.hostname
        port = options.port
        deploy_path = options.deploy_path
        vncviewer = options.vncviewer
        enable_https = options.enable_https
        disable_https = options.disable_https
                
        
        if hostname is None and port is None and deploy_path is None and not enable_https and not disable_https  and vncviewer is None:
            ConfigFile = OvmConfigFile()
            ConfigFile.config()
        ConfigFile = OvmConfigFile(init=False) 
        if hostname is not None:          
            ConfigFile.set("hostname", hostname)
        if port is not None:
            ConfigFile.set("port", port)
        
        if deploy_path is None:
            try:
                ConfigFile.get('deploy_path')
            except Exception, e:
                deploy_path=""
                ConfigFile.set("deploy_path", deploy_path)
            
        if deploy_path is not None:
            ConfigFile.set("deploy_path", deploy_path)
        if enable_https:
            ConfigFile.set("https", 'enabled')
        if disable_https:
            ConfigFile.set("https", 'disabled')
            
        if vncviewer is None:
            try:
                ConfigFile.get('vncviewer')
            except Exception:
                vncviewer=""
                ConfigFile.set("vncviewer", vncviewer)
            
        if vncviewer is not None:
            if vncviewer !="" and not os.path.exists(vncviewer):
                print _("File \"%(name)s\" does not exist.") % {"name":vncviewer}
                return OVM_FAIL
            ConfigFile.set("vncviewer", vncviewer)
            
        print "Configuration complete."
        sys.exit(0)

    def _print_group_help(self, group_name):
        if group_name == 'all':
            print _('Subcommands:\n')
            for group in self.cmd.get_all_groups():
                msgs = self._get_group_help(group)
                for msg in msgs:
                    print msg
            self._print_misc_help()
            self._print_foot_help()
            return True
                
        msgs = self._get_group_help(group_name)
        if msgs == []:
            return False
        desc = self.cmd.get_group_desc_by_name(group_name)
        if not desc:
            desc = group_name
        print _('%(desc)s:\n' % {"desc":desc})
        for msg in msgs:
            print msg
        self._print_foot_help()
        return True

    def _print_misc_help(self):
        standalone = self.cmd.get_standalone_cmd_infos()
        cmds = standalone.keys()
        cmds.sort()
        #print _("\nList of Misc subcommands:")
        for info in cmds:
            print (CONSTANTS.HELP_MSG_FORMAT % (info, standalone[info]))
            
    def _get_group_help(self, group_name):
        cmd_list = self.cmd.get_cmd_list()  
        return [(CONSTANTS.HELP_MSG_FORMAT % (group_name + " " + s['subcmd'], s['desc'])) for s in cmd_list[group_name] if not s.has_key('is_subgroup')]

    def _get_subgroup_help(self, group_name, subgroup_name):
        cmd_list = self.cmd.get_cmd_list()
        return [(CONSTANTS.HELP_MSG_FORMAT % (group_name + " " + s['subcmd'], s['desc'])) for s in cmd_list[group_name] if s.has_key('parent') and s['parent'] == subgroup_name]

    def _print_subgroup_help(self, cmd):
        (group_name, subgroup_name) = cmd.split('_')            
        msgs = self._get_subgroup_help(group_name, subgroup_name)
        if msgs == []:
            return False
        print _('Subcommands:\n')
        for msg in msgs:
            print msg
        self._print_foot_help()
        return True
                                
    def _print_foot_help(self):
        if self.shell:
            print _(
"""\n\"help <subcommand>\" displays help message for that subcommand.
\"help all\" displays complete list of subcommands.
""")

    def _get_cmd(self, args, silent = False):      
        group = args[0]      
        index = self._get_opt_index(args)
        if not index:
            str_cmd = ' '.join(args[1:])
            args = []
        else:
            str_cmd = ' '.join(args[1:index])
            args = args[index:]                           
        subcmd = None
        if group == 'help':
            #print str_cmd.split()
            self.do_help(str_cmd.split())
            return (None, None)  
        if len(str_cmd) != 0:
            if not self.cmd.is_group(group):
                if not silent:
                    print _("Unknown subcommand: %(cmd)s." %{"cmd":group + " " + str_cmd})
                return (None, None)  
            
            subcmd = self.cmd.get_subcmd_by_group(group, str_cmd)
            disp_group = group
            if subcmd is None:
                if not silent:
                    print _("Unknown subcommand: %(cmd)s." %{"cmd":disp_group + " " + str_cmd, "group":disp_group})
                return (None, None)            
            
        if group is None:
            cmd = subcmd
        else:
            cmd = group
            if subcmd:       
                cmd += "_" + subcmd
                
        cmd = cmd.replace(' ', '_')
        return (cmd, args)
         
    def _get_opt_index(self, list):
        for op in list:
            if op.startswith('-'):
                return list.index(op)
        return None

    def _get_complete_names(self):
        names = self.cmd.get_all_groups()
        names.extend(self.cmd.get_all_subcmds())
        names.extend(self.cmd.get_standalone_cmds())
        return names
      
    def _check_serverpool(self, serverPools, serverpool_name = None, silent = False):
        if serverPools is None:
            return False
        if len(serverPools) == 0:
            if serverpool_name is None:
                if silent == False:
                    self.log(_('Server pool not found.'))
            else:
                if silent == False:
                    self.log( _('Server pool "%(serverpool_name)s" does not exist.') % {"serverpool_name":serverpool_name})
            return False
        return True
    
    def _check_ha(self, serverpool):
        if not self.spm.checkHAAbility4ServerPoolById(serverpool._serverPoolId):
            self.log(_("The Server pool \"%(name)s\" does not have HA enabled. \nPlease check the server pool configuration." % {"name":serverpool._serverPoolName}))
            return False
        else:
            return True
        
    def _check_masterserver(self, masterServer):
        if masterServer is None:
            self.log(_("Server pool master does not exist."))
            return False
        return True
    
    def _is_exist_serverpool(self, serverpool_name):
        sp = self._get_serverpool_by_name(serverpool_name, silent = True)
        if sp is None:
            return False
        else:
            return True

    def _get_serverpool_by_name(self, serverpool_name, silent = False):
        # return ServerPool[]
        serverPool = self.spm.getServerPoolByName(serverpool_name)
        if serverPool is None:
           if silent == False:
                self.log(_('Server pool "%(serverpool_name)s" does not exist.') % {"serverpool_name":serverpool_name})
           return None
        return [serverPool]
                
    def _get_all_serverpools(self):
        # return ServerPool[]
        serverPools = self.spm.getAllServerPools()
        if serverPools is None:
            return None
        if len(serverPools) == 0:
            self.log(_('Server pool not found.'))
            return None
        return serverPools
        
    def _get_all_os(self):
        os_types = self.vmm.getAllOperatingSystem()
        if os_types is None:
            self.log(_("Operating system not found."))
            return None
        return os_types


    def _get_server_by_name(self, serverpool_name, server_name):
        serverPools = self._get_serverpool_by_name(serverpool_name)        
        if serverPools is None:
            return None
        server = self.spm.getVirtualServerByName(serverpool_name, server_name)
        if server is None:
            self.log(_('Server "%(name)s" does not exist, or an error occurred while getting server info.') % {"name":server_name})
            return None
        return server
    
    def _get_vm_by_name(self, serverpool_name, vm_name):
        serverPools    = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return None
        vm  = self.vmm.getVMByName(serverpool_name, vm_name)
        if vm is None:
            self.log(_('Virtual machine "%(name)s" does not exist.') %{"name":vm_name})
        return vm        

    def _create_from_template(self, preferred_servers, template_serverpool_name, template_name, 
                              vm_name, console_password, ha_enabled):        
        template = self.rsm.getTemplateResourceByName(template_serverpool_name, template_name)
        if template is None:
            self.log(_("Template \"%(name)s\" does not exist.") %{"name":template_name})
            return OVM_NONE_ERROR
        if template._status != CONSTANTS.ACTIVE:
            self.log(_('Only template in status %(status)s can be used.' % {"status":CONSTANTS.ACTIVE}))
            return OVM_FAIL                
        # Form a NetworkType 
        networkType = self.datamodel.NetworkType()
        networkType._networkType = CONSTANTS.NETWORK_TYPE_AUTO
              
        # Form a NetworkConfType 
        networkConfType = self.datamodel.NetworkConfType() 
        networkConfType._type = CONSTANTS.NETWORK_CONF_TYPE_DYNAMIC 
        # Form a NICConfig
        
        network = self.datamodel.NICConfig()
        network._name     = "vif0"
        network._bridge   = "xenbr0"
        network._rateLimit  = 0
        network._qosEnabled = False
        #network._macAddress = genMac()
        network._networkType = networkType
        network._networkConfType = networkConfType
        
        conf = self.datamodel.TemplateVMConfig()
        conf._vmName               = vm_name
        conf._templateName         = template_name
        conf._serverPoolName       = template_serverpool_name
        conf._haEnable             = ha_enabled
        conf._consolePassword      = console_password 
        conf._vif                  = [network]
        conf._preferredServers     = preferred_servers 
        resultvm = self.lfm.createVmBasedOnTemplate(conf)
        if resultvm is not None and resultvm._imgId is not None:
            self.log(_("Virtual machine \"%(name)s\" is being created. Please check the status.") %{"name":vm_name})
            return OVM_OK
        else:
            return OVM_FAIL

        
    def _create_from_iso(self, serverpool, preferred_servers, iso_serverpool_name, iso_group, iso_name, location, 
                         vm_name, cpu_num, memory_size, disk_size, ha_enabled, console_password, hvm, os):
        iso = None
        if hvm:
            iso = self._get_iso_by_name(iso_serverpool_name, iso_group, iso_name)
            if iso is None or iso._isoResourceId is None:
                self.log(_('ISO image \"%(name)s\" not found in group %(group)s.') % {"name":iso_name, "group":iso_group})
                return OVM_NONE_ERROR
            if iso._status != CONSTANTS.ACTIVE:
                self.log(_('Only ISO images in status %(status)s can be used.' % {"status":CONSTANTS.ACTIVE}))
                return OVM_FAIL
        # Form a NetworkType 
        networkType = self.datamodel.NetworkType()
        networkType._networkType = CONSTANTS.NETWORK_TYPE_AUTO
              
        # Form a NetworkConfType 
        networkConfType = self.datamodel.NetworkConfType() 
        networkConfType._type = CONSTANTS.NETWORK_CONF_TYPE_DYNAMIC 
        # Form a NICConfig
        
        network = self.datamodel.NICConfig()
        network._name     = "vif0"
        network._bridge   = "xenbr0"
        network._rateLimit  = 0
        network._qosEnabled = False
        #network._macAddress = genMac()
        network._networkType = networkType
        network._networkConfType = networkConfType
        
        # Form a DiskMode
        diskMode = self.datamodel.DiskMode()
        diskMode._diskMode = CONSTANTS.DISK_MODE_WRITE
        hardDriveType = self.datamodel.HardDriveType()
        hardDriveType._hardDriveType = CONSTANTS.HARD_DRIVER_AUTO
        # Form a DiskConfig      
        disk = self.datamodel.DiskConfig()
        disk._diskName      = "System"
        disk._diskSize      = disk_size
        disk._priorityClass = 0
        disk._sharable      = False
        #disk._qosEnable     = False
        disk._mode  = diskMode
        disk._hardDriveType = hardDriveType
        
        # Form a BootDevice
        #bootDevice = BootDevice()
        #bootDevice._bootDevice  = CONSTANTS.BOOT_DEVICE_HDD
        # Form a VmType
        vmType = self.datamodel.VmType()
        vmType._vmType  = CONSTANTS.VM_TYPE_PVM
        if hvm:
            vmType._vmType  = CONSTANTS.VM_TYPE_HVM

        # Form a ISOVMConfig
        conf = self.datamodel.ISOVMConfig()
        conf._haEnable                = ha_enabled
        conf._vmType               = vmType
        conf._vmName               = vm_name
        conf._cpuNumber            = cpu_num
        conf._memory               = memory_size
        conf._vncPassword          = console_password
        conf._operatingSystemName  = os
        #conf._vm                   = vm
        conf._serverPool           = serverpool
        conf._diskConf             = disk
        conf._vif                  = [network]
        if hvm:
            conf._iso              = iso
        else:
            conf._isoLocation      =  location
        conf._preferredServers     = preferred_servers
        
        # Create a VM 
        resultvm = self.lfm.createVmBasedOnISO(conf) 
        if resultvm is not None and resultvm._imgId is not None:
            self.log(_("Virtual machine \"%(name)s\" is being created. Please check the status.") %{"name":vm_name})
            return OVM_OK
        else:
            return OVM_FAIL
            
    def _create_from_pxe(self, serverpool, preferred_servers, vm_name, cpu_num, 
                         memory_size, disk_size, ha_enabled, console_password, hvm, os):
        # Form a NetworkType 
        networkType = self.datamodel.NetworkType()
        networkType._networkType = CONSTANTS.NETWORK_TYPE_AUTO
              
        # Form a NetworkConfType 
        networkConfType = self.datamodel.NetworkConfType() 
        networkConfType._type = CONSTANTS.NETWORK_CONF_TYPE_DYNAMIC 
        # Form a NICConfig
        
        network = self.datamodel.NICConfig()
        network._name     = "vif0"
        network._bridge   = "xenbr0"
        network._rateLimit  = 0
        network._qosEnabled = False
        #network._macAddress = genMac()
        network._networkType = networkType
        network._networkConfType = networkConfType
        
        # Form a DiskMode
        diskMode = self.datamodel.DiskMode()
        diskMode._diskMode = CONSTANTS.DISK_MODE_WRITE
        hardDriveType = self.datamodel.HardDriveType()
        hardDriveType._hardDriveType = CONSTANTS.HARD_DRIVER_AUTO
        # Form a DiskConfig      
        disk = self.datamodel.DiskConfig()
        disk._diskName      = "System"
        disk._diskSize      = disk_size
        disk._priorityClass = 0
        disk._sharable      = False
        #disk._qosEnable     = False
        disk._mode  = diskMode
        disk._hardDriveType = hardDriveType

        # Form a VmType
        vmType = self.datamodel.VmType()
        vmType._vmType  = CONSTANTS.VM_TYPE_PVM
        if hvm:
            vmType._vmType  = CONSTANTS.VM_TYPE_HVM
        
        # Form a PXEVMConfig
        conf = self.datamodel.PXEVMConfig()
        #conf._vm                   = vm
        conf._haEnable             = ha_enabled
        conf._vmType               = vmType
        conf._vmName               = vm_name
        conf._cpuNumber            = cpu_num
        conf._memory               = memory_size
        conf._vncPassword          = console_password
        conf._operatingSystemName  = os
        conf._serverPool           = serverpool
        conf._diskConf             = disk
        conf._vif                  = [network]
        conf._preferredServers     = preferred_servers
        # Create a VM 
        resultvm = self.lfm.createVmBasedOnPXE(conf)
        if resultvm is not None and resultvm._imgId is not None:
            self.log(_("Virtual machine \"%(name)s\" is being created. Please check the status.") %{"name":vm_name})
            return OVM_OK
        else:
            return OVM_FAIL           

    def _get_iso_by_name(self, serverpool_name, group_name, iso_name):
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return None
        iso = self.rsm.getISOResourceByGroupNameAndISOName(serverpool_name, group_name, iso_name)
        return iso
            
    def _check_os_type(self, os_name):
        os_types = self._get_all_os()
        if os_types is None:
            return False
        for os_type in os_types:
            if os_type._osName == os_name:
                return True
        print _("Operating system type \"%(name)s\" does not exist.") %{"name":os_name}
        print _("Available operating system types:") 
        for os_type in os_types:
            print os_type._osName
        return False

    def _get_serverpool_name(self, options, silent = False):        
        if options.serverpool_name is not None:
            return options.serverpool_name
        if os.environ.has_key(CONSTANTS.SERVERPOOL_NAME):
            serverpool_name = os.environ[CONSTANTS.SERVERPOOL_NAME]
            if serverpool_name == "":
                if silent == False:
                    print _("%s is required.") % "option -s/--serverpool_name"
                return None
            return serverpool_name
        else:
            if silent == False:
                print _("%s is required.") % "option -s/--serverpool_name"
            return None

    def _get_user_site_names(self, user_id):
        ids = self.adm.get_user_site_ids(user_id)
        result = []
        for id in ids:
            sp = self.spm.getServerPoolById(id)
            if sp is None:
                continue
            result.append(sp._serverPoolName)
        return result

    def _get_groupid_by_name(self, groupname):                 
        groupid = self.adm.get_groupid_by_name(groupname)
        if groupid is None:
            self.log(_("Group \"%(groupname)s\" does not exist." %{"groupname":groupname} ))
            return None
        return groupid

    def _get_ha_status(self, ha):
        if ha is None:
            return ""
        if ha:
            return CONSTANTS.HA_ENABLED
        else:
            return CONSTANTS.HA_DISABLED               

    def _get_login_userid(self, username):
        return self.adm.get_userid_by_name(username)

    
    def _get_vm_status(self, vm):
        status = vm._status._status
        return status
    
    def _get_resource_status(self, status, show_progress = False):
        if show_progress:
            return status
        status_regxp = re.compile("(.*):([\S\s]*)")
        match = status_regxp.search(status)
        if match is not None:
            status = match.group(1)
        return status

    def _get_associated_server(self, vm):
        if vm._status._status != CONSTANTS._RUNNING:
            return ""
        
        vs = self.vmm.getAssociatedServer(vm)
             
        if vs is None or vs._ip is None:
            associated_server = ""
        else:
            associated_server = vs._serverName
        return associated_server
    
    def _show_server_type(self, server_types):
        types = server_types.split(',')
        show_types = []
        for type in types:
            if type == CONSTANTS.MASTER_SERVER:
                show_types.append('Server Pool Master')
                continue
            if type == CONSTANTS.VIRTUAL_SERVER:
                show_types.append('Virtual Machine Server')
                continue
            if type == CONSTANTS.UTILITY_SERVER:
                show_types.append('Utility Server')
                continue
        return ",".join(show_types)

    def _check_session(self):
        if self.session is None and self.shell:
            self.log(_("Not connected."))
            return False
        return True

    def _check_vcpus(self, vcpus):
        if vcpus not in range(1,33):
            self.log(_('VCPU must be between 1 and 32.'))
            return False
        return True

    def _check_sp_name_rules(self, name, value):
        RULES = re.compile("^[\w\-\.]+$")
        if not RULES.match(value):
            print _("The %(name)s is not valid." % {"name":name})
            return False
        return True

    def _check_vm_name_rules(self, name, value):
        if self.OVM_VERSION =='ovm215':
            RULES = re.compile("^[a-zA-Z0-9_]+$")
        else:
            RULES = re.compile("^[a-zA-Z0-9_-]+$")
            
        maxlen = 48
        if len(value) > maxlen:
            print _("The %(name)s must be %(maxlen)d characters or less." % {"name":name, "maxlen":maxlen})
            return False
        if not RULES.match(value):
            print _("The %(name)s is not valid." % {"name":name})
            return False
        return True

    def _check_disk_name_rules(self, name, value):
        RULES = re.compile("^[a-zA-Z0-9-_]+$")
        maxlen = 200
        if len(value) > maxlen:
            print _("The %(name)s must be %(maxlen)d characters or less." % {"name":name, "maxlen":maxlen})
            return False
        if not RULES.match(value):
            print _("The %(name)s is not valid." % {"name":name})
            return False
        return True

    def _check_iso_name_rules(self, name, value):
        if self.OVM_VERSION =='ovm215':
            RULES = re.compile("^[a-zA-Z0-9_-]+$")
        else:
            RULES = re.compile("^[a-zA-Z0-9_][\s\.a-zA-Z0-9_-]*")
        maxlen = 100
        if len(value) > maxlen:
            print _("The %(name)s must be %(maxlen)d characters or less." % {"name":name, "maxlen":maxlen})
            return False
        if not RULES.match(value):
            print _("The %(name)s is not valid." % {"name":name})
            return False
        return True

    def _check_disk_size(self, serverpool_name, name, disk_size):
        if disk_size < 1024:
            self.log(_("The size of a %(name)s must be at least 1024 MB." %{"name":name}))
            return False
        
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return False
        sp = serverPools[0]       
        metrics = self.spm.getServerPoolMetricsByServerPoolId(sp._serverPoolId)
        if metrics:
            free_size = metrics._freeDiskSpace
            if free_size and disk_size > free_size:
                self.log(_("The largest %(name)s that can be allocated is %(size)d MB." %{"name":name,"size":free_size}))
                return False
        else:
            self.log(_("Unable to determine the free disk space on server pool %(sp)s." %{"sp":serverpool_name}))
            return False
        return True
    
    def _compatible_msg(self, type, workversion, name = None):
        if type == 'O':
            type = 'option'
        elif type == 'C':
            type = 'command'
        if name == None:
            name = "This" 
        self.log(_("%(name)s %(type)s requires Oracle VM Manager %(version)s or above." % {"name": name, "type": type, "version":workversion}))

    def _required_server(self, type, workversion, name = None):
        if type == 'O':
            type = 'option'
        elif type == 'C':
            type = 'command'
        if name == None:
            name = "This" 
        self.log(_("%(name)s %(type)s requires Oracle VM Server %(version)s or above." % {"name": name, "type": type, "version":workversion}))

                 
    def _update_object(self, username, password):
        self.username = username
        self.password = password
        self.session = 1
        self.auth=(AUTH.httpbasic, self.username, self.password)
        from CommonPort import CommonPort
        cmm = CommonPort(urlbase=self.urlbase, 
                                       auth = self.auth, 
                                       transport = self.transport) 
        OVM_VERSION = cmm.getOVMVersion()
        if not OVM_VERSION:
            sys.exit()
        self.OVM_VERSION = 'ovm'+OVM_VERSION.replace('.', '')
        #self.OVM_VERSION = 'ovm215'
        exec("import zsigen."+ self.OVM_VERSION + ".datamodel as d")        
        self.datamodel = d
        
        self.spm = ServerPoolManagementPort(urlbase=self.urlbase, 
                                             auth = self.auth, 
                                             transport = self.transport, 
                                             ovm_version = self.OVM_VERSION)
        self.vmm = VirtualMachineManagementPort(urlbase=self.urlbase, 
                                                auth = self.auth, 
                                                transport = self.transport,
                                                ovm_version = self.OVM_VERSION)
        self.rsm = ResourceManagementPort(urlbase=self.urlbase, 
                                          auth = self.auth, 
                                          transport = self.transport,
                                          ovm_version = self.OVM_VERSION)
        self.lfm = LifecycleManagementPort(urlbase=self.urlbase, 
                                           auth = self.auth, 
                                           transport = self.transport,
                                           ovm_version = self.OVM_VERSION)
        self.adm = AdminManagementPort(urlbase=self.urlbase, 
                                       auth = self.auth, 
                                       transport = self.transport,
                                       ovm_version = self.OVM_VERSION)
     
                                          
    def do_val_os(self, options):
        """ val_os --- List all the available Operating Systems"""

        os_types = self._get_all_os()
        result = []
        if os_types is None:
            return OVM_NONE_OS
        for os_type in os_types:
            result.append(os_type._osName)
            self.log(os_type._osName)
        return result

    def do_val_kb(self, options):

        keyboardLayouts = self.lfm.getKeyboardLayoutArray()
        FORMAT = "%-10s %-40s"
        self.log(FORMAT % ("Name", "Description"))
        for kbl in keyboardLayouts:
            self.log(FORMAT %(kbl._value, kbl._name))


    def do_val_bootdev(self, options):
        boot_device_list = [CONSTANTS.BOOT_DEVICE_CDROM, CONSTANTS.BOOT_DEVICE_HDD, CONSTANTS.BOOT_DEVICE_PXE]
        for device in boot_device_list:
            self.log(device)

                           
    def do_vm_ls(self, options):
        """ vm_ls --- Get a list of VMs"""
        vm_name         = options.vm_name       
        serverpool_name = self._get_serverpool_name(options, silent = True)
        show_server = options.show_server
           
        result = []    
        if serverpool_name is None:
            serverPools = self._get_all_serverpools()
        else:
            serverPools = self._get_serverpool_by_name(serverpool_name)
        
        if serverPools is None:
            return result
        
        data = []
        if show_server:     
            data.append(('Name', 'Size(MB)', 'Mem', 'VCPUs', 'Status', 'Server', 'Server_Pool'))
        else:
            data.append(('Name', 'Size(MB)', 'Mem', 'VCPUs', 'Status', 'Server_Pool'))

        for serverpool in serverPools:
            if vm_name is None:
                vms = self.vmm.getAllVMs(serverpool)
                if vms is None:
                    continue           
                for vm in vms:
                    result.append((serverpool._serverPoolName, vm._imgName))
                    if show_server:
                        data.append((vm._imgName, vm._imgSize, vm._memory, vm._cpuNumber, vm._status._status, \
                        self._get_associated_server(vm), serverpool._serverPoolName ))
                    else:
                        data.append((vm._imgName, vm._imgSize, vm._memory, vm._cpuNumber, vm._status._status, \
                        serverpool._serverPoolName ))
                                                  
            else:
                vm = self.vmm.getVMByName(serverpool._serverPoolName, vm_name, silent = True)
                if vm is None:
                    continue
                result.append((serverpool._serverPoolName, vm._imgName))
                if show_server:
                    data.append((vm._imgName, vm._imgSize, vm._memory, vm._cpuNumber, vm._status._status, \
                    self._get_associated_server(vm), serverpool._serverPoolName ))
                else:
                    data.append((vm._imgName, vm._imgSize, vm._memory, vm._cpuNumber, vm._status._status, \
                    serverpool._serverPoolName ))
                                                  
        if len(data) == 1:
            self.log(_('Virtual machine not found.'))
        else:
            self.log(indent(data, sep=self.sep))
 
        return result 
    
  
                                                        
    def do_vm_new(self, options):
        """ vm_new --- Create a VM"""
        method          = options.method
        ha_enabled      = options.ha_enabled
        hvm             = options.hvm
        vm_name         = options.vm_name
        console_password = options.console_password
        preferred_server = options.preferred_server
        
        if not self._check_vm_name_rules('VirtualMachine Name', vm_name):
            return OVM_OPTION_ERROR
        
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
                
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        serverpool = serverPools[0]
        
        if ha_enabled:
            if not self._check_ha(serverpool):
                return OVM_FAIL
            
        preferred_servers = []
        if preferred_server is None:
            preferred_servers = []
        else:
            server_names = preferred_server.split(',')
            for server_name in server_names:
                server_name = server_name.strip()
                server = self._get_server_by_name(serverpool_name, server_name)
                if server is None:
                    return OVM_OPTION_ERROR
                preferred_servers.append(server)
        method = method.lower()         
        if method == "template":                    
            #template_serverpool_name = options.source_serverpool_name  
            #if template_serverpool_name is None:
            #   template_serverpool_name = serverpool_name
            template_serverpool_name = serverpool_name     
            template_name            = options.template
            if template_name is None:
                print _("%s is required for method 'template'.") % "option -t/--template_name"
                return OVM_OPTION_ERROR
            if console_password is None:
                console_password = getpass(_("Console password: ")) 
                                   
            return self._create_from_template(preferred_servers, template_serverpool_name, template_name, 
                                       vm_name, console_password, ha_enabled)
        elif method == "iso":
            #iso_serverpool_name = options.source_serverpool_name
            #if iso_serverpool_name is None:
            #    iso_serverpool_name = serverpool_name
            iso_serverpool_name = serverpool_name
            iso_name           = options.iso_label
            location       = options.location
            iso_group      = options.iso_group
            
            if iso_group is None and hvm:
                print _("%s is required for method 'iso' when hardware virtualization is enabled.") % "option -g/--iso_group"
                return OVM_OPTION_ERROR
            if iso_name is None and hvm:
                print _("%s is required for method 'iso' when hardware virtualization is enabled.") % "option -l/--iso_label"
                return OVM_OPTION_ERROR
            if location is None and not hvm:
                print _("%s is required for method 'iso' when paravirtualization is enabled.") % "option -L/--location"
                return OVM_OPTION_ERROR
            if not hvm and not (location.startswith("http://")   \
                                or location.startswith("ftp://") \
                                or location.startswith("nfs:")):
                print _('"Location" should start with "http://" or "ftp://" or "nfs:"')
                return OVM_OPTION_ERROR                  
            
            os = options.os
            if os is None:
                print _("%s is required for method 'iso'.") % "option -o/--os"
                return OVM_OPTION_ERROR 
            if not self._check_os_type(os):
                return OVM_OPTION_ERROR
                            
            cpu_num = options.vcpus
            if cpu_num is not None:
                if not self._check_vcpus(cpu_num):
                    return OVM_OPTION_ERROR
                
            memory_size         = options.memory
            if memory_size is None:
                print _("%s is required for method 'iso'.") % "option -y/--memory"
                return OVM_OPTION_ERROR 
            disk_size           = options.disksize
            if disk_size is None:
                print _("%s is required for method 'iso'.") % "option -d/--disksize"
                return OVM_OPTION_ERROR
            
            if not self._check_disk_size(serverpool_name, "Virtual Disk", disk_size):
                return OVM_OPTION_ERROR
            
            if console_password is None:
                console_password = getpass(_("Console password: "))    
            return self._create_from_iso(serverpool, preferred_servers, iso_serverpool_name, iso_group, iso_name,
                                  location, vm_name, cpu_num, memory_size, disk_size, ha_enabled, 
                                  console_password, hvm, os)
        elif method == "pxe":
            os = options.os
            if os is None:
                print _("%s is required for method 'pxe'.") % "option -o/--os"
                return OVM_OPTION_ERROR 
            if not self._check_os_type(os):
                return OVM_OPTION_ERROR
            
            cpu_num = options.vcpus
            if cpu_num is not None:
                if not self._check_vcpus(cpu_num):
                    return OVM_OPTION_ERROR
                
            memory_size         = options.memory
            if memory_size is None:
                print _("%s is required for method 'pxe'.") % "option -y/--memory"
                return OVM_OPTION_ERROR 
            disk_size           = options.disksize
            if disk_size is None:
                print _("%s is required for method 'pxe'.") % "option -d/--disksize"
                return OVM_OPTION_ERROR
            
            if not self._check_disk_size(serverpool_name, "Virtual Disk", disk_size):
                return OVM_OPTION_ERROR
            
            if console_password is None:
                console_password = getpass(_("Console password: "))
            return self._create_from_pxe(serverpool, preferred_servers, vm_name, 
                                  cpu_num, memory_size, disk_size, ha_enabled, 
                                  console_password, hvm, os)
        else:
            print _("Unknown method.")
            return OVM_OPTION_ERROR
        
    def do_vm_info(self, options):
        """ vm_info --- Get a VM info """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        
        associated_server = self._get_associated_server(vm)

        prefer_servers =  self.vmm.queryPreferredServerArray(vm._imgId)
        prefer_server = ""
        if prefer_servers == []:
           prefer_server = "Auto"
        if prefer_servers is not None and prefer_servers != []:
            servers = []

            for server in prefer_servers:
                servers.append(server._serverName)
            prefer_server = ",".join(servers)
                         
        group_name = ""
        bootdevice = ""
        keyboard   = ""
        group = self.lfm.getVMGroupByVMId(vm._imgId)
        if group is not None:
            group_name  = group._groupName 
        
        if vm._bootDevice is not None:
            if vm._bootDevice._bootDevice is not None:
                bootdevice =  vm._bootDevice._bootDevice
        if vm._keyboardLayout is not None:
            keyboard = vm._keyboardLayout
        
        pvdriver = vm._pvdriver
        if pvdriver is None:
            pvdriver = "" 
        vm_type = ""
        if vm._virtualMachineType is not None:
            vm_type = vm._virtualMachineType._vmType
            if vm_type is None:
                vm_type = ""
                
        desc = vm._comments
        if desc is None:
            desc = ""
                                
        self.log('%30s %s' % ("ID:", vm._imgId))
        self.log('%30s %s' % ("Virtual Machine Name:", vm._imgName))
        self.log('%30s %s' % ("Virtual Machine Type:", vm_type))
        self.log('%30s %s' % ("Operating System:", vm._operatingSystemName))
        self.log('%30s %s' % ("Status:", vm._status._status))
        self.log('%30s %s' % ("Running on:", associated_server))
        self.log('%30s %s' % ("Size(MB):", vm._imgSize))
        self.log('%30s %s' % ("Memory Size(MB):", vm._memory))
        self.log('%30s %s' % ("Maximum Memory Size(MB):", vm._maximumMemory))
        self.log('%30s %s' % ("Virtual CPUs:", vm._cpuNumber))
        if self.OVM_VERSION != 'ovm215':
            cpupriority = vm._cpuPriority
            cpucap      = vm._cpuCap
            if cpupriority is None:
                cpupriority = ""
            elif cpupriority == 1:
                cpupriority = "Low"
            elif cpupriority == 50:
                cpupriority = "Intermediate" 
            elif cpupriority == 100:
                cpupriority = "High" 
                
            if cpucap is None:
                cpucap = ""
            elif cpucap == 10:
                cpucap = "Low"
            elif cpucap == 50:
                cpucap = "Intermediate" 
            elif cpucap == 100:
                cpucap = "High" 
                
            self.log('%30s %s' % ("VCPU Scheduling Priority:", cpupriority))
            self.log('%30s %s' % ("VCPU Scheduling Cap:", cpucap)) 
                    
        self.log('%30s %s' % ("Boot Device:", bootdevice))
        self.log('%30s %s' % ("Keyboard Layout:", keyboard))
        self.log('%30s %s' % ("Hign Availability:", self._get_ha_status(vm._haEnable)))
        self.log('%30s %s' % ("PVDriver Initialized:", pvdriver))
        self.log('%30s %s' % ("Preferred Server:", prefer_server))
        self.log('%30s %s' % ("Group Name:", group_name))
        self.log('%30s %s' % ("Description:", desc))
        return OVM_OK

    def do_vm_stat(self, options):
        """ vm_stat --- Get a VM status """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        reset          = options.reset                  #   VM name
        # group                                             #   Group Name(not used)
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        if reset:
            if self.lfm.resetStatus(vm):
                self.log(_("Virtual machine status has been reset."))
                return OVM_OK
            else:
                return OVM_FAIL
        else:
            status = self._get_vm_status(vm)
            self.log(status)
            return [status]           
                                                    
    def do_vm_clone(self, options):
        """ vm_clone --- Clone a VirtualMachine """
        source_serverpool_name = self._get_serverpool_name(options)
        if source_serverpool_name is None:
            return OVM_OPTION_ERROR
        
        source_vm_name           = options.source_vm_name                   #   Source VM name
        target_vm_prefix         = options.target_vm_prefix                 #   Target Machine Name Prefix

        if not self._check_vm_name_rules('Target VirtualMachine Name Prefix', target_vm_prefix):
            return OVM_OPTION_ERROR
        
        if options.target_serverpool_name is None:
            target_serverpool_name = source_serverpool_name
        else:
            target_serverpool_name = options.target_serverpool_name       #   Target Server Pool Name
        num_of_copy              = options.num_of_copy                      #   Number of Copies
        group_name               = options.group
        
        group = self.adm.get_group_by_name(group_name)
        if group is None:
            self.log(_("Group \"%(groupname)s\" does not exist." %{"groupname":group_name} ))
            return OVM_FAIL
      
        source_vm  = self._get_vm_by_name(source_serverpool_name, source_vm_name)
        if source_vm is None:
            return OVM_NONE_VM
        status = source_vm._status._status        
        if status != CONSTANTS._POWEREDOFF:
            self.log(_("Only powered off virtual machines can be cloned."))
            return OVM_NOT_READY
        target_serverpools  = self._get_serverpool_by_name(target_serverpool_name)
        if target_serverpools is None:
            return OVM_NONE_SERVERPOOL
        cloneVmConfig     = self.datamodel.CloneVmConfig()
        cloneVmConfig._sourceImg      = source_vm
        cloneVmConfig._vmName = target_vm_prefix
        cloneVmConfig._targetServerPool = target_serverpools[0]
        cloneVmConfig._copyNum = num_of_copy
        cloneVmConfig._targetGroup = group
        if not self.lfm.clone(cloneVmConfig):
            return OVM_FAIL
            
        self.log(_("Cloning."))
        return OVM_OK

    def do_vm_deploy(self, options):
        """ vm_deploy --- Deploy a VirtualMachine """
        
        source_serverpool_name = self._get_serverpool_name(options)
        if source_serverpool_name is None:
            return OVM_OPTION_ERROR
        
        source_vm_name           = options.source_vm_name                   #   Source VM name                #   Target Machine Name Prefix
        target_vm_name           = options.target_vm_name
        
        if not self._check_vm_name_rules('Target VirtualMachine Name', target_vm_name):
            return OVM_OPTION_ERROR
        
        if options.target_serverpool_name is None:
            target_serverpool_name = source_serverpool_name
        else:
            target_serverpool_name = options.target_serverpool_name       #   Target Server Pool Name
        target_group_name = options.target_group                      #   Number of Copies
        
        target_group = self.adm.get_group_by_name(target_group_name)
        if target_group is None:
            self.log(_("Target group \"%(name)s\" does not exist." %{"name":target_group_name}))
            return OVM_NONE_ERROR
      
        source_vm  = self._get_vm_by_name(source_serverpool_name, source_vm_name)
        if source_vm is None:
            return OVM_NONE_VM
        status = source_vm._status._status        
        if status != CONSTANTS._POWEREDOFF:
            self.log(_("Only powered off virtual machines can be deployed."))
            return OVM_NOT_READY
        target_serverpools  = self._get_serverpool_by_name(target_serverpool_name)
        if target_serverpools is None:
            return OVM_NONE_SERVERPOOL
        deployVmConfig     = self.datamodel.DeployVmConfig()
        deployVmConfig._sourceVm      = source_vm
        deployVmConfig._vmName = target_vm_name
        deployVmConfig._targetServerPool = target_serverpools[0]
        deployVmConfig._targetGroup = target_group
        if not self.lfm.deploy(deployVmConfig):
            return OVM_FAIL
            
        self.log(_("Deploying."))
        return OVM_OK
        
    def do_vm_mig(self, options):
        """ vm_mig --- Live Migration"""
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name            = options.vm_name             
        target_server   = options.target_server
        
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        if vm._status._status != CONSTANTS._RUNNING:
            self.log(_("Virtual machine \"%(name)s\" is not ready for migration.") % {"name":vm_name})
            return OVM_NOT_READY
        if target_server is None:
            #self.lfm.setPreferredServers(vm, None)
            if self.lfm.liveMigrate(vm):
                self.log(_("Migrating."))
                return OVM_OK
            else:
                return OVM_FAIL             
        else:
            server = self._get_server_by_name(serverpool_name, target_server)
            if server is None:
                return OVM_NONE_SERVER
            #self.lfm.setPreferredServers(vm, None)
            if self.lfm.liveMigrateToServer(vm, server):
                self.log(_("Migrating."))
                return OVM_OK
            else:
                return OVM_FAIL

    def do_vm_mig_all(self, options):
        """ vm_mig_all --- Migrate all the VMs on the server"""
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        server_name          = options.server_name             
        target_server_name   = options.target_server
        target_server = None
        source_server = self._get_server_by_name(serverpool_name, server_name)
        if source_server is None:
            return OVM_NONE_SERVER
        if target_server_name is not None:
            target_server = self._get_server_by_name(serverpool_name, target_server_name)
            if target_server is None:
                return OVM_NONE_SERVER
        
        if target_server is None:
            if self.lfm.liveMigrateAll(source_server):
                self.log(_("Migrating."))
                return OVM_OK
            else:
                return OVM_FAIL
        else:
            if self.lfm.liveMigrateAllToServer(source_server, target_server):
                self.log(_("Migrating."))
                return OVM_OK
            else:
                return OVM_FAIL        
        
    def do_vm_del(self, options):
        """ vm_del --- Delete a VirtualMachine """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #  VM name
        safe             = options.safe
        
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        if safe:
            if self.vmm.unregisterVM(vm._imgId):
                self.log(_("Deleted virtual machine \"%(name)s\".") % {"name":vm_name})
                return OVM_OK
            else:
                return OVM_FAIL
        else:
            if self.vmm.deleteVM(vm._imgId):
                self.log(_("Deleted virtual machine \"%(name)s\" and its image files.") % {"name":vm_name})
                return OVM_OK
            else:
                return OVM_FAIL
                

    def do_vm_save(self, options):
        """ vm_save --- Save a VirtualMachine as template """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        template_name     = options.template_name
        # group                                             #   Group Name(not used)
        if not self._check_vm_name_rules('Template Name', template_name):
            return OVM_OPTION_ERROR
        
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        if vm._status._status == CONSTANTS._POWEREDOFF:
            if template_name is None:
                template_name = vm_name + "_tem"
            discover_templates = self.rsm.getAllDiscoverableTemplates(serverpool_name)
            if discover_templates is not None:
                for template in discover_templates:
                    if template._imgName == template_name:     
                        self.log(_("Discoverable template \"%(name)s\" already exists. Please specify another template name." 
                                   % {"name":template_name}))
                        return OVM_FAIL
            if self.rsm.getTemplateResourceByName(serverpool_name, template_name):
                self.log(_("Template \"%(name)s\" already exists. Please specify another template name."  % {"name":template_name}))
                return OVM_FAIL
            
            if self.vmm.saveAsTemplate(vm, template_name):
                self.log(_("Saving virtual machine as a template. Please check the status."))
                return OVM_OK
            else:
                return OVM_FAIL
        else:
            self.log(_("Only powered off virtual machines can be saved as templates."))
            return OVM_NOT_READY    
                        
    def do_vm_poweron(self, options):
        """ vm_poweron --- PowerOn a VirtualMachine """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        force            = options.force
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
            
        status = self._get_vm_status(vm)
        if status == CONSTANTS._POWEREDOFF or force:
            if self.vmm.powerOnVM(vm._imgId):
                self.log(_("Powering on."))
                return OVM_OK
            else:
                return OVM_FAIL
        else:
            self.log(_("Only powered off virtual machines can be powered on."))
            return OVM_NOT_READY
                    
    def do_vm_reboot(self, options):
        """ vm_reboot --- Restart a VirtualMachine """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        # group                                             #   Group Name(not used)
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        status = self._get_vm_status(vm)
        if status == CONSTANTS._RUNNING:
            if self.vmm.rebootVM(vm._imgId):
                self.log(_("Rebooting."))
                return OVM_OK
            else:
                return OVM_FAIL
        else:
            self.log(_("Only running virtual machines can be rebooted."))
            return OVM_NOT_READY
                
    def do_vm_pause(self, options):
        """ vm_pause --- Pause a VirtualMachine """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        # group                                             #   Group Name(not used)
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        # check the status first
        status = self._get_vm_status(vm)
        if status == CONSTANTS._RUNNING:
           if self.vmm.pauseVM(vm._imgId):
                self.log(_("Pausing."))
                return OVM_OK
           else:
                return OVM_FAIL
        else:
            self.log(_("Only running virtual machines can be paused."))
            return OVM_NOT_READY
                
    def do_vm_unpause(self, options):
        """ vm_unpause --- Unpause a VirtualMachine """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        # group                                             #   Group Name(not used)
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        # check the status first
        status = self._get_vm_status(vm)
        if status == CONSTANTS._PAUSED:
            if self.vmm.unpauseVM(vm._imgId):
                self.log(_("Unpausing."))
                return OVM_OK
            else:
                return OVM_FAIL
        else:
            self.log(_("Only paused virtual machines can be unpaused."))
            return OVM_NOT_READY
              
    def do_vm_suspend(self, options):
        """ vm_suspend --- Suspend a VirtualMachine """

        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        # group                                             #   Group Name(not used)
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        # check the status first
        status = self._get_vm_status(vm)
        if status == CONSTANTS._RUNNING:
            if self.vmm.suspendVM(vm._imgId):
                self.log(_("Suspending."))
                return OVM_OK
            else:
                return OVM_FAIL
        else:
            self.log(_("Only running virtual machines can be suspended."))
            return OVM_NOT_READY
            
    def do_vm_resume(self, options):
        """ vm_resume --- Resume a VirtualMachine """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        # group                                             #   Group Name(not used)
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        # check the status first
        status = self._get_vm_status(vm)
        if status == CONSTANTS._SUSPENDED:
            if self.vmm.resumeVM(vm._imgId):
                self.log(_("Resuming."))
                return OVM_OK
            else:
                return OVM_FAIL
        else:
            self.log(_("Only suspended virtual machines can be resumed."))
            return OVM_NOT_READY
            
    def do_vm_poweroff(self, options):
        """ vm_poweroff --- Poweroff a VirtualMachine """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        force            = options.force
        
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        status = self._get_vm_status(vm)
        if status == CONSTANTS._RUNNING or status == CONSTANTS._UNKNOWN or force:
            if self.vmm.powerOffVM(vm._imgId):
                self.log(_("Shutting down."))
                return OVM_OK
            else:
                return OVM_FAIL
        else:
            self.log(_("Only running virtual machines can be powered off."))
            return OVM_NOT_READY
                                        
    def do_vm_conf(self, options):
        """ vm_conf --- Config a VirtualMachine """

        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        new_name         = options.new_name
        max_mem          = options.new_maxmem
        memory           = options.new_memory
        vcpus            = options.new_vcpus
        os               = options.new_os
        # add in 2.2
        new_priority     = options.new_priority
        new_cap          = options.new_cap
        new_group        = options.new_group
        # end
        enable_ha        = options.enable_ha
        disable_ha       = options.disable_ha
        preferred_servers = options.preferred_servers
        preferred_servers_auto = options.preferred_servers_auto
        network_type           = options.network_type
        description      = options.new_description
        
        if vcpus is not None:
            if not self._check_vcpus(vcpus):
                return OVM_OPTION_ERROR
        if new_priority is not None:
            if new_priority not in range(1,101):
                self.log(_('New VCPU scheduling priority must be between 1 and 100.'))
                return OVM_OPTION_ERROR

        if new_cap is not None:
            if new_cap not in range(10,101):
                self.log(_('New VCPU scheduling cap must be between 10 and 100.'))
                return OVM_OPTION_ERROR        
            
        if preferred_servers_auto and preferred_servers is not None:
            self.log(_("preferred_servers and preferred_servers_auto cannot be specified at the same time."))
            return OVM_OPTION_ERROR
        
        if enable_ha and disable_ha:
            self.log(_("enable_ha and disable_ha cannot be specified at the same time."))
            return OVM_OPTION_ERROR            

        if network_type is not None and network_type not in [CONSTANTS.NETWORKTYPE_IOEMU, CONSTANTS.NETWORKTYPE_NETFRONT]:
            self.log(_("Only network types %(ioemu)s and %(netfront)s are allowed." 
                           %{"ioemu":CONSTANTS.NETWORKTYPE_IOEMU, "netfront":CONSTANTS.NETWORKTYPE_NETFRONT}))
            return OVM_OPTION_ERROR
                                      
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        if new_name is not None:
            if not self._check_vm_name_rules('New VirtualMachine Name', new_name):
                return OVM_OPTION_ERROR
            if self.vmm.setVMName(vm, new_name):
                self.log(_("Virtual machine name changed to \"%(name)s\".") %{"name":new_name})
            else:
                return OVM_FAIL

        if max_mem is not None:
            if self.lfm.setMaximumMemory(vm, max_mem):
                self.log(_("Maximum Memory Size has been changed to \"%(max_mem)s\".") %{"max_mem":max_mem})
            else:
                return OVM_FAIL

        if memory is not None:
            if self.lfm.setDynamicMemory(vm, memory):
                self.log(_("Memory size changed to \"%(memory)s\".") %{"memory":memory})
            else:
                return OVM_FAIL
                
        if vcpus is not None:
            if self.lfm.setCpuCores(vm, vcpus):
                self.log(_("Number of VCPUs changed to \"%(vcpus)s\".") %{"vcpus":vcpus})
            else:
                return OVM_FAIL
        
        if new_priority:
            if self.OVM_VERSION == 'ovm215':
                self._compatible_msg('O', '2.2', name = "-r/--new_priority")
            else:
                if self.lfm.setVCPUPriorityByVMID(vm._imgId, new_priority):
                    self.log(_("VCPU scheduling priority changed to \"%(num)s\".") %{"num":new_priority})
                else:
                    return OVM_FAIL

        if new_cap:
            if self.OVM_VERSION == 'ovm215':
                self._compatible_msg('O', '2.2', name = "-C/--new_cap")
            else:
                if self.lfm.setVCPUCapByVMID(vm._imgId, new_cap):
                    self.log(_("VCPU scheduling cap changed to \"%(num)s\".") %{"num":new_cap})
                else:
                    return OVM_FAIL

        if new_group:
            if self.OVM_VERSION == 'ovm215':
                self._compatible_msg('O', '2.2', name = "-g/--new_group")
            else:
                group_id = self.adm.get_groupid_by_name(new_group)
                if not group_id:
                    self.log(_("Group \"%(groupname)s\" does not exist." %{"groupname":new_group} ))
                    return OVM_FAIL
                
                if self.lfm.configVMGroup(vm._imgId, group_id):
                    self.log(_("Group changed to \"%(groupname)s\".") %{"groupname":new_group})
                else:
                    return OVM_FAIL
                                        
        if enable_ha:
            if self.lfm.enableHA(vm):
                self.log(_("High availability has been enabled."))
            else:
                return OVM_FAIL
        if disable_ha:
            if self.lfm.disableHA(vm):
                self.log(_("High availability has been disabled."))
            else:
                return OVM_FAIL
        if description is not None:
            if self.lfm.setVMDescription(vm, description):
                self.log(_("Description changed to \"%(desc)s\".") % {"desc":description})
            else:
                return OVM_FAIL
                
        if os is not None:
            if not self._check_os_type(os):
                return OVM_NONE_OS
            os_type = self.datamodel.OperatingSystem()
            os_type.OsId = 111
            os_type._osName = os
            if self.lfm.setOperatingSystem(vm, os_type):
                self.log(_("Operating system changed to \"%(os)s\".") %{"os":os})
            else:
                return OVM_FAIL
            
        if preferred_servers_auto:
            if self.lfm.setPreferredServers(vm, []):
                self.log(_("Preferred server mode set to \"Auto\"."))
            else:
                return OVM_FAIL
                            
        if preferred_servers is not None:
            servers = preferred_servers.split(',')
            preferred_list = []
            for server_name in servers:
                server = self._get_server_by_name(serverpool_name, server_name.strip())
                if server is None:
                    return OVM_NONE_SERVER
                preferred_list.append(server)                
            if self.lfm.setPreferredServers(vm, preferred_list):
                self.log(_("Preferred server configured."))
            else:
                return OVM_FAIL
            
        if network_type is not None:
            networkType = self.datamodel.NetworkType()
            networkType._networkType = network_type
            if self.lfm.changeNetworkType(vm, networkType):
                self.log(_("Network type set to \"%(type)s\"." %{"type":network_type}))
            else:
                return OVM_FAIL                                                                      
            
        return OVM_OK

    def do_vm_disk_add(self, options):
        """ vm_disk_add --- Create and Add a disk to the VM """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        disk_name        = options.disk_name
        disk_size        = options.disk_size
        priorityclass    = options.priority_class
        enable_qos       = options.enable_qos
        drive_type       = options.drive_type

        if not self._check_disk_name_rules('Disk Name', disk_name):
            return OVM_OPTION_ERROR

        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        
        if not self._check_disk_size(serverpool_name, "Virtual Disk", disk_size):
            return OVM_OPTION_ERROR
                             
        if priorityclass < 0 or priorityclass > 7:
            self.log(_("Priority class must be between 0 and 7."))
            return OVM_OPTION_ERROR
      
        drive_types = [CONSTANTS.HARD_DRIVER_AUTO, 
                       CONSTANTS.HARD_DRIVER_IDE, 
                       CONSTANTS.HARD_DRIVER_SCSI,
                       CONSTANTS.HARD_DRIVER_XVD]
        if drive_type not in drive_types:
            self.log(_("Hard drive type specified is invalid."))
            self.log(_("Available hard drive types:"))
            for type in drive_types:
                self.log(type)
            return OVM_OPTION_ERROR
        
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM

        vm_type = ""
        if vm._virtualMachineType is not None:
            vm_type = vm._virtualMachineType._vmType
        if vm_type == CONSTANTS.VM_TYPE_HVM and drive_type == CONSTANTS.HARD_DRIVER_XVD:
            self.log(_("Cannot add XVD disks to a hardware virtualized guest."))
            return OVM_OPTION_ERROR
                             
        hardDriveType = self.datamodel.HardDriveType()
        hardDriveType._hardDriveType = drive_type
        
        conf = self.datamodel.DiskConfig()
        conf._diskName = disk_name
        conf._diskSize = disk_size   
        conf._qoSEnable = enable_qos
        conf._priorityClass = priorityclass
        conf._hardDriveType = hardDriveType
        #conf._hardDiskDriver = drive_type
        vd = self.lfm.addDisk(vm, conf)
        #print vd.__dict__
        if vd is not None:
            self.log(_("Adding virtual disk. Please check the status."))
            return OVM_OK
        else:
            return OVM_FAIL

    def do_vm_disk_conf(self, options):
        """ vm_disk_conf --- Config a disk of the VM """
        if self.OVM_VERSION == 'ovm215':
            self._compatible_msg('C', '2.2')
            return OVM_FAIL
        
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        disk_name        = options.disk_name
        priorityclass    = options.priority_class
        enable_qos       = options.enable_qos
        disable_qos      = options.disable_qos
        drive_type       = options.drive_type
        
        if not (priorityclass or enable_qos or disable_qos or drive_type):
            self.log(_("Found nothing to update."))
            return OVM_OPTION_ERROR
        
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        if priorityclass:
            if priorityclass < 0 or priorityclass > 7:
                self.log(_("Priority class must be between 0 and 7."))
                return OVM_OPTION_ERROR
        if drive_type:
            drive_types = [CONSTANTS.HARD_DRIVER_AUTO, 
                           CONSTANTS.HARD_DRIVER_IDE, 
                           CONSTANTS.HARD_DRIVER_SCSI,
                           CONSTANTS.HARD_DRIVER_XVD]
            if drive_type not in drive_types:
                self.log(_("Hard drive type specified is invalid."))
                self.log(_("Available hard drive types:"))
                for type in drive_types:
                    self.log(type)
                return OVM_OPTION_ERROR
        
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        if drive_type:
            vm_type = ""
            if vm._virtualMachineType is not None:
                vm_type = vm._virtualMachineType._vmType
            if vm_type == CONSTANTS.VM_TYPE_HVM and drive_type == CONSTANTS.HARD_DRIVER_XVD:
                self.log(_("Cannot change the hard drive type to XVD because it is a hardware virtualized guest."))
                return OVM_OPTION_ERROR
                             
        disks = self.vmm.queryLocalDisks(vm)
        for disk in disks._item:       
            if disk._diskName == disk_name:
                virtualDisk = self.datamodel.VirtualDisk()
                virtualDisk = disk
                if drive_type:
                    hardDriveType = self.datamodel.HardDriveType()
                    hardDriveType._hardDriveType = drive_type
                    virtualDisk._hardDiskDriver = hardDriveType
                if priorityclass:
                    virtualDisk._priorityClass = priorityclass
                if enable_qos:
                    virtualDisk._qosEnable = True
                if disable_qos:
                    virtualDisk._qosEnable = False
                  
                if self.lfm.configDisk(vm._imgId, virtualDisk):
                    self.log(_("Virtual disk updated."))
                    return OVM_OK
                else:
                    return OVM_FAIL
        self.log(_("Virtual disk \"%(name)s\" does not exist." ) % {"name":disk_name})
        return OVM_FAIL


    def do_vm_disk_del(self, options):
        """ vm_disk_del --- Remove a disk from the VM """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        disk_name        = options.disk_name
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        disks = self.vmm.queryLocalDisks(vm)
        for disk in disks._item:       
            if disk._diskName == disk_name:
                if self.lfm.removeDisk(disk):
                    self.log(_("Removed virtual disk from the virtual machine."))
                    return OVM_OK
                else:
                    return OVM_FAIL
        self.log(_("Virtual disk \"%(name)s\" does not exist." ) % {"name":disk_name})
        return OVM_FAIL
        
    def do_vm_disk_ls(self, options):
        """ vm_disk_ls --- List Disks of the VM """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        result = []
        disks = self.vmm.queryLocalDisks(vm)
        if disks is not None:
            result.extend(disks._item)
        sds = self.vmm.querySharedVirtualDiskArray(vm._imgId)
        if sds is not None:
            result.extend(sds)
                      
        data = []
        if self.OVM_VERSION != 'ovm215':
            data.append(('Disk_Name', 'Size(MB)', 'Hard_Disk_Driver', 'Shared', 'QoS', 'Attachment', 'Status'))
        else:
            data.append(('Disk_Name', 'Size(MB)', 'Shared', 'QoS', 'Attachment', 'Status'))
        for disk in result:
            attached = disk._attachStatus
            status   = disk._status
            sharable = disk._sharable
            qos      = disk._qosEnable
            if attached is None:
                attached = ""
            if status is None:
                status = ""            
            if disk.__dict__.has_key('_sharedDiskResourceId'):
                sharable = "Sharable"
            else:
                sharable = "Non-Sharable"
            if qos:
                qos = "Y"
            else:
                qos = "N"

            priorityclass = disk._priorityClass
            if priorityclass is not None:
                qos = qos + ":" + str(priorityclass)
            if self.OVM_VERSION != 'ovm215':
                disk_type = disk._hardDiskDriver._hardDriveType
                if disk_type is None:
                    disk_type = ""
                elif disk_type == 'sd':
                    disk_type = 'SCSI'
                elif disk_type == 'hd':
                    disk_type = 'IDE'
                elif disk_type == 'xvd':
                    disk_type = 'XVD'
                elif disk_type == 'auto':
                    disk_type = 'AUTO'
                data.append((disk._diskName, disk._diskSize, disk_type, sharable, qos, attached , status))    
            else:                                                                        
                data.append((disk._diskName, disk._diskSize, sharable, qos, attached , status))
        self.log(indent(data, sep=self.sep))
        return result
        
    def do_vm_nic_add(self, options):
        """ vm_nic_add --- Create and Add a nic to the VM """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        nic_name         = options.nic_name
        bridge           = options.bridge
        rate_limit       = options.rate_limit
        enable_qos       = options.enable_qos

        if not self._check_disk_name_rules('NIC Name', nic_name):
            return OVM_OPTION_ERROR
                
        if rate_limit < 0:
            self.log(_("Rate limit cannot be negative."))
            return OVM_OPTION_ERROR
        
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM

        serverPools = self._get_serverpool_by_name(serverpool_name)
        bridges = self.spm.getNetworkBridges(serverPools[0])
        bridge_devices = []
        for device in  bridges._mapEntry:
            bridge_devices.append(device._key)
        if bridge not in bridge_devices:
            if bridge is not None:
                self.log(_("The network bridge specified does not exist."))
            else:
                print _("%s is required.") % "option -b/--bridge"
            if bridge_devices == []:
                self.log(_("No network bridges found."))
            else:
                 self.log(_("Available network bridges:"))
                 for device_name in bridge_devices:
                     self.log(device_name)
                 return OVM_FAIL
                        
        networkType = self.datamodel.NetworkType()
        networkType._networkType = CONSTANTS.NETWORK_TYPE_AUTO
              
        # Form a NetworkConfType 
        networkConfType = self.datamodel.NetworkConfType() 
        networkConfType._type = CONSTANTS.NETWORK_CONF_TYPE_DYNAMIC 
        # Form a NICConfig
        
        network = self.datamodel.NICConfig()
        network._name     = nic_name
        network._bridge   = bridge
        network._rateLimit  = rate_limit
        network._qosEnabled = enable_qos
        network._networkType = networkType
        network._networkConfType = networkConfType
        
        nic = self.lfm.addNIC(vm, network)
        if nic is not None:
            self.log(_("NIC has been added to the virtual machine."))
            return OVM_OK
        else:
            return OVM_FAIL                                        

    def do_vm_nic_del(self, options):
        """ vm_nic_del --- Remove a nic from the VM """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        nic_name        = options.nic_name
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        nics = self.vmm.queryNetworkInterfaceCards(vm)
        if nics is None:
            return OVM_FAIL
        for nic in nics:       
            if nic._name == nic_name:               
                if self.lfm.removeNIC(nic):
                    self.log(_("NIC has been deleted from the virtual machine."))
                    return OVM_OK
                else:
                    return OVM_FAIL
        self.log(_("NIC \"%(name)s\" does not exist." ) % {"name":nic_name})
        return OVM_FAIL

    def do_vm_nic_conf(self, options):
        """ vm_nic_conf --- Config a nic of the VM """

        if self.OVM_VERSION == 'ovm215':
            self._compatible_msg('C', '2.2')
            return OVM_FAIL
        
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        nic_name         = options.nic_name
        new_nic_name     = options.new_nic_name
        bridge           = options.bridge
        rate_limit       = options.rate_limit
        enable_qos       = options.enable_qos
        disable_qos      = options.disable_qos

        if not (new_nic_name or bridge or (rate_limit is not None) or enable_qos or disable_qos):
            self.log(_("Found nothing to update."))
            return OVM_OPTION_ERROR
          
        if new_nic_name and not self._check_disk_name_rules('New NIC Name', new_nic_name):
            return OVM_OPTION_ERROR
                
        if rate_limit is not None and rate_limit < 0:
            self.log(_("Rate limit cannot be negative."))
            return OVM_OPTION_ERROR

        if enable_qos and disable_qos:
            self.log(_("enable_qos and disable_qos cannot both be specified."))
            return OVM_OPTION_ERROR
                
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
    
        if bridge:
            serverPools = self._get_serverpool_by_name(serverpool_name)
            bridges = self.spm.getNetworkBridges(serverPools[0])
            bridge_devices = []
            for device in  bridges._mapEntry:
                bridge_devices.append(device._key)
            if bridge not in bridge_devices:
                if bridge is not None:
                    self.log(_("The network bridge specified does not exist."))
                else:
                    print _("%s is required.") % "option -b/--bridge"
                if bridge_devices == []:
                    self.log(_("No network bridges found."))
                else:
                     self.log(_("Available network bridges:"))
                     for device_name in bridge_devices:
                         self.log(device_name)
                     return OVM_FAIL                    
        
        nics = self.vmm.queryNetworkInterfaceCards(vm)
        if nics is None:
            return OVM_FAIL
        for nic in nics:       
            if nic._name == nic_name:
                network = self.datamodel.VirtualNetworkCard()
                network = nic
                if new_nic_name:
                    network._name     = new_nic_name
                if bridge:
                    network._bridge   = bridge
                if rate_limit is not None:
                    network._rateLimit  = rate_limit
                if enable_qos:
                    network._qosEnabled = True
                if disable_qos:
                    network._qosEnabled = False
         
                if self.lfm.configNIC(vm._imgId, network):
                    self.log(_("NIC updated."))
                    return OVM_OK
                else:
                    return OVM_FAIL
        self.log(_("NIC \"%(name)s\" does not exist." ) % {"name":nic_name})
        return OVM_FAIL
                  
        
    def do_vm_nic_ls(self, options):
        """ vm_nic_ls --- List Virtual Network Interfaces of the VM """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        nics = self.vmm.queryNetworkInterfaceCards(vm)
        if nics is None:
            return []
        data = []
        data.append(('NIC_Name', 'MAC_Adress', 'Bridge', 'Status', 'Interface', "Rate_Limit"))
        for nic in nics:
            network_type = ""    
            mac = ""
            bridge = ""
            if nic._networkType is not None:
                network_type = nic._networkType._networkType
            if nic._macAddress is not None:
                mac = nic._macAddress
            if nic._bridge is not None:
                bridge = nic._bridge
            qos = nic._qosEnabled 
            ratelimit = nic._rateLimit
            if ratelimit is not None:
                qos = str(qos) + ":" + str(ratelimit)
            if qos is None:
                qos = ''                            
            data.append((nic._name, mac, bridge, nic._networkStatus._networkStatus, network_type, qos))                                  
        if len(data) == 1:
            self.log('NIC not found.')
        else:
            self.log(indent(data, sep=self.sep))
        return nics
    
    def do_vm_sd_attach(self, options):
        """ vm_sd_attach --- Attach a Shared Virtual Disk to the VM """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        disk_name        = options.disk_name
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        
        sd = self.rsm.getSharedDiskResourceByName(serverpool_name, disk_name)
        if sd is None:
            self.log(("Shared virtual disk \"%(name)s\" does not exist.")%{"name":disk_name})
            return OVM_FAIL
                     
        if sd._status != CONSTANTS.ACTIVE:
            self.log(_('Only shared virtual disks of status %(status)s can be attached.' % {"status":CONSTANTS.ACTIVE}))
            return OVM_FAIL
        vm_group = self.lfm.getVMGroupByVMId(vm._imgId)
        if vm_group:
            if vm_group._groupId != sd._groupId:
                self.log(_('Only shared virtual disks in group \"%(name)s\" can be attached to \"%(vm_name)s\".' % {"name":vm_group._groupName, "vm_name":vm_name}))
                return OVM_FAIL
                
        if self.lfm.attachSharedVirtualDisk(vm, sd):
            self.log(_("Attached shared virtual disk to the virtual machine."))
            return OVM_OK
        else:
            return OVM_FAIL

    def do_vm_sd_detach(self, options):
        """ vm_sd_detach --- Detach a Shared Virtual Disk from the VM """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        diskName         = options.disk_name
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        sds = self.vmm.querySharedVirtualDiskArray(vm._imgId)
        if sds is None:
            return OVM_FAIL
        for sd in sds:
            if sd._diskName == diskName:
                if self.lfm.detachSharedVirtualDisk(vm, sd):
                    self.log(_("Detached shared virtual disk from the virtual machine."))
                    return OVM_OK
                else:
                    return OVM_FAIL
        self.log(_("Shared virtual disk \"%(name)s\" does not exist.") % {"name":diskName})
        return OVM_FAIL                                                                 
    
    def do_vm_cd_attach(self, options):
        """ vm_cd_attach --- Attach a CDROM to the VM """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        iso_group        = options.iso_group
        iso_label        = options.iso_label
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        if vm._virtualMachineType is not None: 
            vm_type = vm._virtualMachineType._vmType
            if vm_type != CONSTANTS.VM_TYPE_HVM:
                self.log(_('CD-ROM operations are only available for hardware virtualized guests.'))
                return OVM_FAIL
        iso = self._get_iso_by_name(serverpool_name, iso_group, iso_label)
        if iso is None:
            self.log(_('ISO image "%(name)s" not found in group \"%(group)s\".') % {"name":iso_label, "group":iso_group})
            return OVM_NONE_ERROR
        
        if iso._status != CONSTANTS.ACTIVE:
            self.log(_('Only ISO images with status %(status)s can be attached.' % {"status":CONSTANTS.ACTIVE}))
            return OVM_FAIL
            
        conf = self.datamodel.CdConfig()
        conf._iso = iso
        if self.lfm.attachCDtoVM(vm, conf):
            self.log(_("Attached CD-ROM to the virtual machine."))
            return OVM_OK
        else:
            return OVM_FAIL

    def do_vm_cd_detach(self, options):
        """ vm_cd_detach --- Detach CDROMs from the VM """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        if vm._virtualMachineType is not None: 
            vm_type = vm._virtualMachineType._vmType
            if vm_type != CONSTANTS.VM_TYPE_HVM:
                self.log(_('CD-ROM operations are only available for hardware virtualized guests.'))
                return OVM_FAIL
                    
        if self.lfm.detachCD(vm):
            self.log(_("Detached CD-ROM from the virtual machine."))
            return OVM_OK
        else:
            return OVM_FAIL

    def do_vm_cd_ls(self, options):
        """ vm_cd_ls --- List CDROMs of the VM """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        if vm._virtualMachineType is not None: 
            vm_type = vm._virtualMachineType._vmType
            if vm_type != CONSTANTS.VM_TYPE_HVM:
                self.log(_('CD-ROM operations are only available for hardware virtualized guests.'))
                return OVM_FAIL
        cdroms = self.vmm.queryCdrom(vm)
        if cdroms is None:
            return []
        if len(cdroms) == 0:
            self.log(_("ISO image not attached."))
            return []
        for cdrom in cdroms:
            if cdrom._iso is not None:
                self.log(cdrom._iso._ISOName)
            else:
                self.log(_("ISO image not attached."))
        
        return cdroms

    def do_vm_bootdev(self, options):
        """ vm_bootdev --- Set the first BootDevice """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name            = options.vm_name                  #   VM name
        boot_device        = options.boot_device
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        boot_device_list = [CONSTANTS.BOOT_DEVICE_CDROM, CONSTANTS.BOOT_DEVICE_HDD, CONSTANTS.BOOT_DEVICE_PXE]
        if boot_device not in boot_device_list:
            self.log(_("Specified boot device is not valid."))
            self.log(_("Available boot devices:"))
            for device in boot_device_list:
                self.log(device)
            return OVM_FAIL
        
        bootDevice = self.datamodel.BootDevice()
        bootDevice._bootDevice = boot_device
        if self.lfm.setBootPriority(vm, bootDevice):
            self.log(_("Boot device set to %(name)s." % {"name":boot_device}))
            return OVM_OK
        else:
            return OVM_FAIL

    def do_vm_vncpwd(self, options):
        """ vm_vncpwd --- Set the VNC Console Password """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name            = options.vm_name                  #   VM name
        console_password           = options.console_password
        if console_password is None:
            console_password = getpass(_("New console password: ")) 
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        if self.lfm.setConsolePassword(vm, console_password):
            self.log(_("Updated VNC console password."))
            return OVM_OK
        else:
            return OVM_FAIL


    def do_vm_kb(self, options):
        """ vm_kb --- Set the Keyboard Layout """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name            = options.vm_name                  #   VM name
        keyboard           = options.keyboard
         
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        
        keyboardLayouts = self.lfm.getKeyboardLayoutArray()
        keyboardLayout = None
        if keyboard is not None:        
            for kbl in keyboardLayouts:
                if kbl._value == keyboard:
                    keyboardLayout = kbl
                    break
        if keyboardLayout is None:
            if keyboard is None:
                print _("%s is required.") % "option -k/--keyboard" 
            self.log(_("Available keyboard layouts:"))
            FORMAT = "%-10s %-40s"
            self.log(FORMAT % ("Name", "Description"))
            for kbl in keyboardLayouts:
                self.log(FORMAT %(kbl._value, kbl._name))
            
            return OVM_OPTION_ERROR
       
        if self.lfm.setVMKeyboardLayout(vm._imgId, keyboardLayout):
            self.log(_("Keyboard layout set to \"%(name)s\"." % {"name":keyboard}))
            return OVM_OK
        else:
            return OVM_FAIL
                                    
    def do_svrp_new(self, options):
        """ svrp_new --- Create a ServerPool """
        hostname               = options.hostname
        server_name            = options.server_name
        vip                    = options.vip
        agent_password         = options.agent_password
        server_location        = options.server_location
        description            = options.description
        utility_username       = options.utility_username
        utility_password       = options.utility_password
        ha_enabled             = options.ha_enabled
        serverpool_name        = options.serverpool_name
        user_list              = options.user_list
        if server_name is None:
            server_name = hostname
        
        if serverpool_name is not None:
            if not self._check_sp_name_rules("ServerPool Name", serverpool_name):
                return OVM_OPTION_ERROR

        if server_name is not None:
            if not self._check_sp_name_rules("Server Name", server_name):
                return OVM_OPTION_ERROR
                         
        ids = []
        if user_list is not None:
            usernames = user_list.split(",")
            ids = self.adm.get_userids_by_names(usernames)
            if type(ids) == type(''):
               self.log(_('User "%(name)s" does not exist.') % {"name":ids})
               return OVM_FAIL
           
        if agent_password is None:
            agent_password = getpass(_("Oracle VM Agent password: "))
              
        if utility_password is None:
            utility_password = getpass(_("Utility server password: "))
        
        user_ids = []   

        # Add non-Admin users
        user_ids.extend(ids)
        # Add all the Admin users      
        user_ids.extend(self.adm.get_all_admin_ids())
        user_ids = uniq(user_ids)                                
        # check if the serverpool exists
        if self._is_exist_serverpool(serverpool_name):
            self.log(_('Server pool "%(name)s" already exists.') % {"name":serverpool_name})
            return OVM_EXIST
        
        masterServer    = self.datamodel.MasterVirtualServer()
        virtualServer   = self.datamodel.VirtualServer()
        utilityServer   = self.datamodel.UtilityVirtualServer()
        # Form a masterServer
        masterServer._serverName  = server_name 
        masterServer._ip          = hostname
        masterServer._agentPwd    = agent_password
        masterServer._serverType  = CONSTANTS.MASTER_SERVER
        if server_location:
            masterServer._location = server_location
        if description:
            masterServer._comments = description
        # Form a virtualServer
        virtualServer._serverName = server_name 
        virtualServer._ip         = hostname
        virtualServer._agentPwd   = agent_password
        virtualServer._serverType = CONSTANTS.VIRTUAL_SERVER
        if server_location:
            virtualServer._location = server_location
        if description:
            virtualServer._comments = description
        # Form a utilityServer
        utilityServer._ip         = hostname
        utilityServer._serverName = server_name 
        utilityServer._agentPwd   = agent_password
        utilityServer._serverType = CONSTANTS.UTILITY_SERVER      
        utilityServer._loginUsername  = utility_username
        utilityServer._loginPassword   = utility_password
        if server_location:
            utilityServer._location = server_location
        if description:
            utilityServer._comments = description
        
        # Form a ServerPoolConfig
        serverPoolConfig = self.datamodel.ServerPoolConfig()    
        serverPoolConfig._HA_enabled         = ha_enabled
        serverPoolConfig._serverPoolName     = serverpool_name
        # Add in 2.2
        serverPoolConfig._serverPoolVirtualIP = vip       
        serverPoolConfig._masterServer       = masterServer
        serverPoolConfig._virtualServers     = serverPoolConfig.new_virtualServers()
        serverPoolConfig._utilityServers     = serverPoolConfig.new_utilityServers()
        serverPoolConfig._virtualServers._item.append(virtualServer)
        serverPoolConfig._utilityServers._item.append(utilityServer)
        
        if not self.spm.validateCreateServerPool(serverPoolConfig):
            return OVM_FAIL
        
        result = self.spm.createServerPool(serverPoolConfig, user_ids)
        if result is not None:
            self.log(_('Created server pool \"%(name)s\".') % {"name":result._serverPoolName})
            return OVM_OK
        else:
            return OVM_FAIL


    def do_svrp_del(self, options):
        """ svrp_del --- Delete a ServerPool """
        serverpool_name           = options.serverpool_name
        isDelWorkingDir           = options.isdelworkdirectory
        force                     = options.force
       
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        serverPool = serverPools[0]
        if self.spm.deleteServerPool(serverPool, 
                                     isDelWorkingDir = isDelWorkingDir, 
                                     force = force):
            msg = _('Server pool "%(name)s" deleted.')
            if force:
                msg = _("Server pool \"%(name)s\" deleted (forced).")
            if isDelWorkingDir:
                msg = _("Server pool \"%(name)s\" deleted, with working directories removed.")
            self.log(msg % {"name":serverPool._serverPoolName})
            return OVM_OK
        else:
            return OVM_FAIL

    def do_svrp_ls(self, options):
        """ svrp_ls --- Get a list of ServerPools """
        result = []
        serverPools = self._get_all_serverpools()
        if serverPools is None:
            return result   
        data = []
        data.append(('Server_Pool_Name', 'Status', 'HA'))
        for sp in serverPools:
            # print sp.__dict__
            result.append(sp._serverPoolName)
            data.append((sp._serverPoolName, sp._status, self._get_ha_status(sp._haEnable)))
        
        self.log(indent(data, sep=self.sep))
        return result
            
    def do_svrp_refresh(self, options):
        """ svrp_refresh --- Refresh all of the ServerPools """
        serverPools = self._get_all_serverpools()
        if serverPools is None:
            return OVM_NONE_SERVERPOOL 
        for sp in serverPools:
            if self.spm.refreshServerPool(sp) is None:
                return OVM_FAIL
        self.log(_("Refresh server pool data."))
        return OVM_OK
                        
    def do_svrp_stat(self, options):
        """ svrp_stat --- Get a ServerPool status """
        
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR       
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL     
        sp = serverPools[0]
        status = sp._status
        self.log(status)
        return [status]
            
    def do_svrp_info(self, options):
        """ svrp_info --- Get a ServerPool info """
        
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
                
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        sp = serverPools[0]       
        metrics = self.spm.getServerPoolMetricsByServerPoolId(sp._serverPoolId)

        totalNumOfCpus  = ""
        totalNumOfChips = ""
        totalCPUSpeedMHZ   = ""
        allocatedVCpu = ""
        totalMemory   = ""
        freeMemory    = ""   
        freeDiskSpace = ""
        
        if metrics is not None:
            totalNumOfCpus  = metrics._totalNumOfCpus
            totalNumOfChips = metrics._totalNumOfCpus
            totalCPUSpeedMHZ   = metrics._totalCPUSpeedMHZ
            allocatedVCpu = metrics._allocatedVCpu
            totalMemory   = metrics._totalMemory
            freeMemory    = metrics._freeMemory   
            freeDiskSpace = metrics._freeDiskSpace    
        self.log('%30s %s' % ("ID:", sp._serverPoolId))
        self.log('%30s %s' % ("Server Pool Name:", sp._serverPoolName))
        # add in 2.2
        if self.OVM_VERSION != 'ovm215':
            vip = sp._serverPoolVirtualIP
            if not vip:
                vip = ""
            self.log('%30s %s' % ("Server Pool Virtual IP:", vip))
        # end
        self.log('%30s %s' % ("Status:", sp._status))
        self.log('%30s %s' % ("High Availability:", self._get_ha_status(sp._haEnable)))
        self.log('%30s %s' % ("Total Number of CPUs:", totalNumOfCpus))
        self.log('%30s %s' % ("Total CPU Speed(MHZ):", totalCPUSpeedMHZ))
        self.log('%30s %s' % ("Total Number of Chips:", totalNumOfChips))
        self.log('%30s %s' % ("Allocated VCPUs:", allocatedVCpu))
        self.log('%30s %s' % ("Total Memory(MB):", totalMemory))
        self.log('%30s %s' % ("Free Memory(MB):", freeMemory))
        self.log('%30s %s' % ("Free Disk Space(MB):", freeDiskSpace))
        return OVM_OK
        
    def do_svrp_restore(self, options):
        """ svrp_restore --- Restore a ServerPool """

        serverpool_name = options.serverpool_name    
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return 1        

        sp = serverPools[0]
        self.log(_("Restoring server pool data from Oracle VM Manager."))
        if self.spm.restoreVirtualServerPoolByManagerData(sp._serverPoolId):
            self.log(_("Server pool data restored successfully."))
            return OVM_OK
        else:
            return OVM_FAIL 
        
    def do_svrp_conf(self, options):
        """ svrp_conf --- Config a ServerPool"""

        serverpool_name           = options.serverpool_name
        new_poolname              = options.new_poolname
        enable_ha                 = options.enable_ha
        disable_ha                = options.disable_ha
        vip                       = options.vip
        
        if new_poolname is None:
            new_poolname          = serverpool_name

        if new_poolname is not None:
            if not self._check_sp_name_rules("New ServerPool Name", new_poolname):
                return OVM_OPTION_ERROR
                    
        if (not enable_ha) and (not disable_ha) and vip is None and serverpool_name == new_poolname:
            print _("Found nothing to update.")
            return OVM_OPTION_ERROR
        
        if self.OVM_VERSION == 'ovm215':
            if vip is not None:
                self._compatible_msg('O', '2.2', name = "-i/--vip") 
                return OVM_OPTION_ERROR               
        
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        sp = serverPools[0]        
        poolId = sp._serverPoolId
        if enable_ha:
            ha_enabled = True
        if disable_ha:
            ha_enabled = False
        if (not enable_ha) and (not disable_ha):
             ha_enabled = sp._haEnable
             
        if self.OVM_VERSION == 'ovm215':
            if (enable_ha == sp._haEnable or 
                disable_ha == (not sp._haEnable)) and serverpool_name == new_poolname:
                self.log(_("Nothing to update."))
                return 1       
            if self.spm.updateServerPool(poolId, ha_enabled, new_poolname):
                self.log(_("Server pool updated."))
                return OVM_OK
            else:
                return OVM_FAIL
            
        if (enable_ha == sp._haEnable or 
            disable_ha == (not sp._haEnable)) and serverpool_name == new_poolname and vip == sp._serverPoolVirtualIP:
            self.log(_("Nothing to update."))
            return 1
        
        if vip is None:
            vip = sp._serverPoolVirtualIP
                   
        if self.spm.updateServerPoolVIP(poolId, ha_enabled, new_poolname, vip):
            self.log(_("Server pool updated."))
            return OVM_OK
        else:
            return OVM_FAIL        
                      
    # Resource Management
            
    def do_tmpl_reg(self, options):
        """ tmpl_reg --- Register a Discoverable Template """
        
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        template_name     = options.template_name            # Virtual Machine Template Name
        os_type           = options.os_type                  # Operating System Type
        username          = options.username                 # Virtual Machine system username
        password          = options.password                 # Virtual Machine system password

        if os_type is not None:
            if not self._check_os_type(os_type):
                return OVM_NONE_OS
                    
        if password is None:
            password = getpass(_("Virtual machine system password: ")) 
            
        description       = options.description              # Description
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        #userid = self._get_login_userid(self.username)
        
        templates = self.rsm.getAllDiscoverableTemplates(serverpool_name)
        for tl in templates:
            if tl._imgName == template_name:                    
                conf = self.datamodel.VirtualMachineTemplate()
                conf._parentImgId    = 0
                conf._deployType     = "Auto"
                conf._imgType        = "Xen"
                conf._serverPoolId   = tl._serverPoolId
                conf._imgName        = tl._imgName
                conf._imgLocation    = "/OVS/seed_pool/" + tl._imgName
                conf._osType          = os_type
                conf._vmUsername      = username
                conf._vmPassword      = password
                conf._description   = description
                conf._vncPort         = 5900
                conf._userId          = self._get_login_userid(self.username)
                #conf._groupId         = CONSTANTS.MYWORKSPACE              
                if self.rsm.registerInternalTemplate(serverpool_name, conf):                    
                    self.log(_("Registering template. Please check the status."))
                    return OVM_OK                    
                else:
                    return OVM_FAIL

        if templates == []:
            self.log(_('Discoverable template not found.'))
            return OVM_FAIL
        
        self.log(_('Template \"%(name)s\" cannot be registered.') % {"name":template_name})
      
        self.log(_('Discoverable templates:'))
        for template in templates:
            self.log(template._imgName)
        return OVM_FAIL

    def do_tmpl_disc(self, options):
        """ tmpl_disc --- List all of the Discoverable Templates"""
    
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        result = []
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL

        templates = self.rsm.getAllDiscoverableTemplates(serverpool_name)
        if templates is None:
            return result

        if templates == []:
            self.log(_('Discoverable template not found.'))
            return result
                     
        self.log(_('Discoverable templates:'))
        for template in templates:
            result.append(template._imgName)
            self.log(template._imgName)
        return result

    def do_tmpl_approve(self, options):
        """ tmpl_approve --- Approve a Template """

        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        template_name    = options.template_name
        
        serverPools = self._get_serverpool_by_name(serverpool_name)
        
        if serverPools is None:
            return OVM_OPTION_ERROR
        templateResource = self.rsm.getTemplateResourceByName(serverpool_name, template_name, silent = True)
        if templateResource is None:
                self.log(_('Template "%(name)s" does not exist, or an error occurred while getting the template resource.') % {"name":template_name})
                return OVM_NONE_TEMPLATE
        if templateResource._status != CONSTANTS.PENDING:
            self.log(_('Only templates with status %(status)s can be approved.' % {"status":CONSTANTS.PENDING}))
            return OVM_FAIL
        
        if self.rsm.approveTemplate(serverpool_name, template_name):
            self.log(_('Template "%(name)s" approved.') % {"name":template_name})
            return OVM_OK
        else:
            return OVM_FAIL
                    
    def do_tmpl_imp(self, options):
        """ tmpl_imp --- Import a Template from an External Source"""
     
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        template_name     = options.template_name            # Virtual Machine Template Name
        os_type           = options.os_type                  # Operating System Type
        username          = options.username                 # Virtual Machine system username
        password          = options.password                 # Virtual Machine system password       
        url               = options.url
        description       = options.description              # Description
        proxy             = options.proxy

        if not self._check_vm_name_rules('Template Name', template_name):
            return OVM_OPTION_ERROR
                    
        if os_type is not None:
            if not self._check_os_type(os_type):
                return OVM_NONE_OS
        if password is None:
            password = getpass(_("Virtual machine system password: "))  
            
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
       # userid = self._get_login_userid(self.username)
        
        template = self.rsm.getTemplateResourceByName(serverpool_name, template_name)

        if template is not None:
            self.log(_("Template \"%(name)s\" already exists.") % {"name":template_name})
            return OVM_EXIST            
        conf = self.datamodel.VirtualMachineTemplate()
        conf._parentImgId    = 0
        conf._imgName        = template_name
        conf._osType          = os_type
        conf._vmUsername      = username
        conf._vmPassword      = password
        conf._description   = description
        conf._vncPort         = 5900
        conf._downloadURL     = url
        conf._userId          = self._get_login_userid(self.username)
        #conf._groupId         = CONSTANTS.MYWORKSPACE 
        if proxy is not None:
            conf._proxyURL   = proxy
            conf._proxyUsed  = True
        
        if self.rsm.registerExternalTemplate(serverpool_name, conf):
            self.log("Downloading template. Please check the status.")
            return OVM_OK
        else:
            return OVM_FAIL   
                         
    def do_tmpl_ls(self, options):
        """ tmpl_ls --- Get a list of Templates """
        serverpool_name = self._get_serverpool_name(options, silent = True)
        
        result = []
        
        if serverpool_name is None:
            serverPools = self._get_all_serverpools()
        else:
            serverPools = self._get_serverpool_by_name(serverpool_name)
        
        if serverPools is None:
            return OVM_NONE_SERVERPOOL 
        data = []
        data.append(("Name", "Size(MB)", "Server_Pool", "Status", "Created"))         
        for sp in serverPools:
            templateResources = self.rsm.getAllTemplateResources(sp._serverPoolName)
            if templateResources is None:
                continue
            for tr in templateResources:
                result.append((tr._serverPoolName, tr._imgName))
                createTime = datetime.datetime.fromtimestamp(tr._createdTime/1000)
                img_size = tr._imgSize
                if img_size is None:
                    img_size = ""
                data.append((tr._imgName, img_size, tr._serverPoolName, tr._status, 
                     createTime.strftime("%Y-%m-%d")))
            
        if len(data) == 1:
            self.log(_('Template not found.'))
        else:
            self.log(indent(data, sep=self.sep))
        return result 
                
    def do_tmpl_del(self, options):
        """ tmpl_del --- Delete a Template """

        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        template_name    = options.template_name
        
        serverPools = self._get_serverpool_by_name(serverpool_name)
        
        if serverPools is None:
            return OVM_OPTION_ERROR
             
        templateResource = self.rsm.getTemplateResourceByName(serverpool_name, template_name)
        if templateResource is None:
            self.log(_('Template "%(name)s" does not exist, or an error occurred while getting the template resource.') % {"name":template_name})
            return OVM_FAIL
        if self.rsm.deleteTemplate(serverpool_name, template_name):
            self.log(_('Template "%(name)s" deleted.') % {"name":template_name})
            return OVM_OK
        else:
            return OVM_FAIL
            

    def do_tmpl_stat(self, options):
        """ tmpl_stat --- Show the template status """

        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        template_name    = options.template_name
        progress    = options.progress
        
        serverPools = self._get_serverpool_by_name(serverpool_name)
        
        if serverPools is None:
            return OVM_OPTION_ERROR
             
        templateResource = self.rsm.getTemplateResourceByName(serverpool_name, template_name)
        if templateResource is None:
            self.log(_('Template "%(name)s" does not exist, or an error occurred while getting the template resource.') % {"name":template_name})
            return OVM_FAIL
        status = self._get_resource_status(templateResource._status, progress)
        self.log(status)
        return [status]
                
    def do_img_imp (self, options):
        """ img_imp --- Import an Image from an External Source"""

        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        image_name        = options.image_name            
        os_type           = options.os_type                  # Operating System Type
        username          = options.username                 # Virtual Machine system username
        password          = options.password                 # Virtual Machine system password
        url               = options.url
        description       = options.description              # Description
        proxy             = options.proxy
        console_password  = options.console_password
        ha_enabled        = options.enable_ha
        group             = options.group

        if not self._check_vm_name_rules('Image Name', image_name):
            return OVM_OPTION_ERROR
            
        if os_type is not None:
            if not self._check_os_type(os_type):
                return OVM_NONE_OS

        if console_password is None:
            console_password = getpass(_("Console password: ")) 
                         
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL

        if ha_enabled:
            if not self._check_ha(serverPools[0]):
                return OVM_FAIL

        groupid = self._get_groupid_by_name(group)
        if groupid is None:
            return OVM_FAIL
        image = self.rsm.getImageResourceByName(serverpool_name, image_name)
        if image is not None:
            self.log(_("Image \"%(name)s\" already exists.") % {"name":image_name})
            return OVM_EXIST 
        vm  = self.vmm.getVMByName(serverpool_name, image_name)           
        if vm is not None:
            self.log(_("Virtual machine \"%(name)s\" already exists.") % {"name":image_name})
            return OVM_EXIST
        conf = self.datamodel.VirtualMachineImage()
        conf._imgName        = image_name
        conf._osType          = os_type
        if username is not None:
            conf._vmUsername      = username
        if password is not None:
            conf._vmPassword      = password
        conf._description   = description
        conf._vncPort         = 5900
        conf._downloadURL     = url
        conf._vncPassword     = console_password
        conf._haEnabled       = ha_enabled
        conf._userId          = self._get_login_userid(self.username)
        conf._groupId         = groupid  
        if proxy is not None:
            conf._proxyURL   = proxy
            conf._proxyUsed  = True
            
            
        if self.rsm.registerExternalImage(serverpool_name, conf):
            self.log("Downloading virtual machine image. Please check the status.")
            return OVM_OK
        else:
            return OVM_FAIL   
                                    
    def do_img_ls (self, options):
        """ img_ls --- Get a list of VM images """
        serverpool_name = self._get_serverpool_name(options, silent = True)
        result = []
        if serverpool_name is None:
            serverPools = self._get_all_serverpools()
        else:
            serverPools = self._get_serverpool_by_name(serverpool_name)

        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        data = []
        data.append(("Name", "Size(MB)", "Server_Pool", "Status", "Created"))         
        for sp in serverPools:
            images = self.rsm.getAllImageResources(sp._serverPoolName)
            if images is None:
                continue
            for vm in images:
                result.append((vm._serverPoolName, vm._imgName))
                createTime = datetime.datetime.fromtimestamp(vm._createdTime/1000)
                img_size = vm._imgSize
                if img_size is None:
                    img_size = ""
                
                data.append((vm._imgName, img_size, vm._serverPoolName, vm._status, 
                     createTime.strftime("%Y-%m-%d")))
        if len(data) == 1:
            self.log(_('Virtual machine image not found.'))
        else:
            self.log(indent(data, sep=self.sep))
        return result 
    
    def do_img_del(self, options):
        """ img_del --- Delete a VM image """

        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        image_name       = options.image_name
        serverPools = self._get_serverpool_by_name(serverpool_name)
        
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        
        imageResource = self.rsm.getImageResourceByName(serverpool_name, image_name)
        if imageResource is None:
            self.log(_('Virtual machine image "%(name)s" does not exist, or an error occurred while getting the image resource.') % {"name":image_name})
            return OVM_FAIL
        if self.rsm.deleteImage(serverpool_name, image_name):
            self.log(_("Virtual machine image \"%(name)s\" deleted.") %{"name":image_name})
            return OVM_OK
        else:
            return OVM_FAIL

    def do_img_stat(self, options):
        """ img_stat --- Show the Image status """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        image_name       = options.image_name
        progress    = options.progress
        serverPools = self._get_serverpool_by_name(serverpool_name)
        
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        
        imageResource = self.rsm.getImageResourceByName(serverpool_name, image_name)
        if imageResource is None:
            self.log(_('Virtual machine image "%(name)s" does not exist, or an error occurred while getting the image resource.') % {"name":image_name})
            return OVM_FAIL
        status = self._get_resource_status(imageResource._status, progress)
        self.log(status)
        return [status]


    def do_img_reg(self, options):
        """ img_reg --- Register a Discoverable VM image """
    
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        image_name             = options.image_name          # Virtual Machine Image Name
        ha_enabled             = options.enable_ha          # Enable high availability
        os_type                = options.os_type             # Operating System
        username               = options.username            # Virtual Machine system username
        password               = options.password            # virtual Machine system password
        console_password       = options.console_password    # Console Password
        description            = options.description         # Description
        group                  = options.group
        if os_type is not None:
            if not self._check_os_type(os_type):
                return OVM_NONE_OS
        if console_password is None:
            console_password = getpass(_("Console password: ")) 

        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL

        if ha_enabled:
            if not self._check_ha(serverPools[0]):
                return OVM_FAIL

        groupid = self._get_groupid_by_name(group)
        if groupid is None:
            return OVM_FAIL
                    
        images= self.rsm.getAllDiscoverableImages(serverpool_name)
        for vm in images:
            if vm._imgName == image_name:                    
                virtualMachineImage = self.datamodel.VirtualMachineImage()
                virtualMachineImage._deployType     = "Auto"
                virtualMachineImage._imgType        = "Xen"
                virtualMachineImage._serverPoolId   = vm._serverPoolId
                virtualMachineImage._imgName         = vm._imgName
                virtualMachineImage._imgLocation    = "/OVS/running_pool/" + vm._imgName
                virtualMachineImage._osType         = os_type
                if username is not None:
                    virtualMachineImage._vmUsername      = username
                if password is not None:
                    virtualMachineImage._vmPassword      = password
                virtualMachineImage._description   = description
                virtualMachineImage._haEnabled       = ha_enabled
                virtualMachineImage._vncPassword     = console_password
                virtualMachineImage._vncPort         = 0
                virtualMachineImage._userId          = self._get_login_userid(self.username)
                virtualMachineImage._groupId         = groupid          
                if self.rsm.registerInternalImage(serverpool_name, virtualMachineImage):
                    self.log(_("Registering virtual machine image. Please check the status."))
                    return OVM_OK          

        if images == []:
            self.log(_('Discoverable virtual machine image not found.'))
            return OVM_FAIL        
        self.log(('Virtual machine image "%(name)s" cannot be registered. ') % {"name":image_name})
        
        self.log(_('Discoverable virtual machine images:'))
        for vm in images:
            self.log(vm._imgName)
        return OVM_FAIL

    def do_img_disc(self, options):
        """ img_disc --- List all of the Discoverable VM images """
    
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        result = []
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL

        images= self.rsm.getAllDiscoverableImages(serverpool_name)
        if images is None:
            return result
        
        if images == []:
            self.log(_('Discoverable virtual machine image not found.'))
            return result
             
        self.log(_('Discoverable virtual machine images:'))
        for vm in images:
            self.log(vm._imgName)
            result.append(vm._imgName)
        return result

    def do_img_approve(self, options):
        """ img_approve --- Approve a VM Image """

        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        image_name    = options.image_name
        
        serverPools = self._get_serverpool_by_name(serverpool_name)
        
        if serverPools is None:
            return OVM_OPTION_ERROR
        imageResource = self.rsm.getImageResourceByName(serverpool_name, image_name)
        if imageResource is None:
                self.log(_('Virtual machine image "%(name)s" does not exist, or an error occurred while getting the image resource.') % {"name":image_name})
                return OVM_NONE_VM

        if imageResource._status != CONSTANTS.PENDING:
            self.log(_('Only virtual machine images with status %(status)s can be approved.' % {"status":CONSTANTS.PENDING}))
            return OVM_FAIL
        
        if self.rsm.approveImage(serverpool_name, image_name):
            self.log(_('Virtual machine image \"%(name)s\" approved.') % {"name":image_name})
            return OVM_OK
        else:
            return OVM_FAIL
                                                       
    def do_iso_reg (self, options):
        """ iso_reg --- Register a Discoverable ISO image """        
        
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        iso_group         = options.iso_group                # ISO Group
        iso_label         = options.iso_label                # ISO label
        description       = options.description              # Description
        
        if description is None:
            description = ""
        
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL  
        iso_exist = False 
    
        isos = self.rsm.getAllDiscoverableISO(serverpool_name)

        if isos == []:
            self.log(_('Discoverable ISO image not found.'))
            return OVM_FAIL
                             
        for iso in isos:
            if iso._ISOLabel == iso_label and iso._ISOGroup == iso_group:
                isoResourceConfig = self.datamodel.ISOResourceConfig()
                isoResourceConfig._serverPoolId  = iso._serverPoolId
                isoResourceConfig._ISOName = iso._ISOLabel
                isoResourceConfig._ISOGroup = iso._ISOGroup
                isoResourceConfig._comments = description
                if self.rsm.registerInternalISO(serverpool_name, isoResourceConfig) is not None:
                    self.log(_("ISO image \"%(label)s\" registered.") % {"label":iso_label})
                    return OVM_OK
                else:
                    return OVM_FAIL
                        
        self.log(_('ISO image \"%(label)s\" cannot be registered.') % {"label":iso_label}) 
        self.log(_('Discoverable ISO images:'))
        FORMAT = "%-10s %-50s"
        if len(isos) > 0:
            self.log(FORMAT %("ISOGroup", "ISOLabel"))
        for iso in isos:
            label = iso._ISOLabel
            group = iso._ISOGroup
            self.log(FORMAT % (group, label))
        return OVM_FAIL

    def do_iso_disc(self, options):
        """ iso_disc --- List all of the Discoverable ISOs"""
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR

        serverPools = self._get_serverpool_by_name(serverpool_name)
        result = []
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        isos = self.rsm.getAllDiscoverableISO(serverpool_name)
        if isos is None:
            return result
        
        if isos == []:
            self.log(_('Discoverable ISO image not found.'))
            return result
                     
        self.log(_('Discoverable ISO images:'))
        FORMAT = "%-10s %-50s"
        if len(isos) > 0:
            self.log(FORMAT %("ISO_Group", "ISO_Label"))
        for iso in isos:
            label = iso._ISOLabel
            group = iso._ISOGroup
            result.append((group, label))
            self.log(FORMAT % (group, label))
        return result 

    def do_iso_approve(self, options):
        """ iso_approve --- Approve an ISO image """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        iso_name         = options.iso_label
        iso_group         = options.iso_group
        serverPools = self._get_serverpool_by_name(serverpool_name)

        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        iso = self._get_iso_by_name(serverpool_name, iso_group, iso_name)
        if iso is None:
            self.log(_('ISO image "%(name)s" does not exist, or an error occurred while getting the ISO image.') % {"name":iso_name})
            return OVM_FAIL
        
        if iso._status != CONSTANTS.PENDING:
            self.log(_('Only ISO images with status %(status)s can be approved.' % {"status":CONSTANTS.PENDING}))
            return OVM_FAIL
        
        if self.rsm.approveISOResource(iso._isoResourceId):
            self.log(_("ISO image \"%(name)s\" approved.") %{"name":iso_name})
            return OVM_OK
        else:          
            return OVM_FAIL
                    
    def do_iso_imp (self, options):
        """ iso_imp --- Import an ISO from External Source"""        
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        iso_group         = options.iso_group                # ISO Group
        iso_label         = options.iso_label                # ISO label
        description       = options.description              # Description
        url               = options.url
        proxy             = options.proxy

        if not self._check_iso_name_rules('ISO Label', iso_label):
            return OVM_OPTION_ERROR

        if not self._check_vm_name_rules('ISO Group', iso_group):
            return OVM_OPTION_ERROR
                                
        iso = self._get_iso_by_name(serverpool_name, iso_group, iso_label)
        if iso is not None:
            self.log(_("ISO image \"%(name)s\" already exists.") % {"name":iso_label})
            return OVM_EXIST
            
        conf = self.datamodel.ISOResourceConfig()
        conf._ISOName        = iso_label
        conf._ISOGroup       = iso_group
        conf._downloadURL     = url
        conf._comments        = description
        if proxy is not None:
            conf._proxyURL   = proxy
        if self.rsm.registerExternalISO(serverpool_name, conf):
            self.log("Downloading ISO image. Please check the status.")
            return OVM_OK
        else:
            return OVM_FAIL   

    def do_iso_ls(self, options):
        """ iso_ls --- Get a list of ISO images """       
        
        serverpool_name = self._get_serverpool_name(options, silent = True)
        
        if serverpool_name is None:
            serverPools = self._get_all_serverpools()
        else:
            serverPools = self._get_serverpool_by_name(serverpool_name)
        
        result = []
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        # ISO_Label ISO_Group FileName Status Server_Pool_Name
        data = []
        data.append(('ISO_Label', 'ISO_Group', 'Status', 'Server_Pool' ))         
        for sp in serverPools:
            isos = self.rsm.getAllISOResources(sp._serverPoolName)
            if isos is None:
                continue
            for iso in isos:
                result.append((sp._serverPoolName, iso._ISOName))
                data.append((iso._ISOName, iso._ISOGroup, iso._status, sp._serverPoolName))
        if len(data) == 1:
            self.log(_('ISO image not found.'))
        else:
            self.log(indent(data, sep=self.sep))
        return result 

    def do_iso_del(self, options):
        """ iso_del --- Delete an ISO image """

        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        iso_name         = options.iso_label
        iso_group         = options.iso_group
        serverPools = self._get_serverpool_by_name(serverpool_name)
        
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
            # TO DO, missing getISOResourceByName
        iso = self._get_iso_by_name(serverpool_name, iso_group, iso_name)
        if iso is None:
            self.log(_('ISO image "%(name)s" does not exist, or an error occurred while getting the ISO image.') % {"name":iso_name})
            return OVM_FAIL
        if self.rsm.deleteISO(iso._isoResourceId):
            self.log(_("ISO image \"%(name)s\" deleted.") %{"name":iso_name})
            return OVM_OK
        else:          
            return OVM_FAIL

    def do_iso_stat(self, options):
        """ iso_stat --- Show the ISO status """

        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        iso_name         = options.iso_label
        iso_group         = options.iso_group
        progress    = options.progress
        serverPools = self._get_serverpool_by_name(serverpool_name)
        
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
            # TO DO, missing getISOResourceByName
        iso = self._get_iso_by_name(serverpool_name, iso_group, iso_name)
        if iso is None:
            self.log(_('ISO image "%(name)s" does not exist, or an error occurred while getting the ISO image.') % {"name":iso_name})
            return OVM_FAIL
        status = self._get_resource_status(iso._status, progress)
        self.log(status)
        return [status]
                                         
    def do_sd_new(self, options):
        """ sd_new --- Create and Register a Shared Virtual Disk """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        disk_name       = options.disk_name
        disk_size       = options.disk_size
        comments        = options.comments
        group           = options.group

        if not self._check_disk_name_rules('Disk Name', disk_name):
            return OVM_OPTION_ERROR
        
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        
        if not self._check_disk_size(serverpool_name, "Shared Virtual Disk", disk_size):
            return OVM_OPTION_ERROR
        
        groupid = self._get_groupid_by_name(group)
        if groupid is None:
            return OVM_FAIL
        conf = self.datamodel.SharedVirtualDiskConfig()
        conf._diskName = disk_name
        conf._diskSize = disk_size
        conf._comments = comments
        conf._userId          = self._get_login_userid(self.username)
        conf._groupId         = groupid
        resource = self.rsm.getSharedDiskResourceByName(serverpool_name, disk_name)
        if resource is not None:
            self.log(_("Shared virtual disk \"%(name)s\" already exists.") % {"name":disk_name})
            return OVM_EXIST
        if self.rsm.registerSharedVirtualDisk(serverpool_name, conf):
            self.log(_("Shared virtual disk \"%(name)s\" is being created. Please check the status.") % {"name":disk_name})
            return OVM_OK
        else:
            return OVM_FAIL
                       
    def do_sd_del(self, options):
        """ sd_del --- Delete a Shared Virtual Disk """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        disk_name         = options.disk_name
        
        serverPools = self._get_serverpool_by_name(serverpool_name)
        
        if serverPools is None:
            return OVM_NONE_SERVERPOOL

        sharedDiskResource = self.rsm.getSharedDiskResourceByName(serverpool_name, disk_name)
        if sharedDiskResource is None:
            self.log(_('Shared virtual disk "%(name)s" does not exist, or an error occurred while getting the shared disk resource.') % {"name":disk_name})
            return OVM_FAIL
            
        if self.rsm.deleteSharedVirtualDisk(sharedDiskResource._sharedDiskResourceId):
            self.log(_("Shared virtual disk \"%(name)s\" deleted.") %{"name":disk_name})
            return OVM_OK
        else:
            return OVM_FAIL


    def do_sd_stat(self, options):
        """ sd_stat --- Show the Shared Virtual Disk status """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        disk_name         = options.disk_name
        
        serverPools = self._get_serverpool_by_name(serverpool_name)
        
        if serverPools is None:
            return OVM_NONE_SERVERPOOL

        sharedDiskResource = self.rsm.getSharedDiskResourceByName(serverpool_name, disk_name)
        if sharedDiskResource is None:
            self.log(_('Shared virtual disk "%(name)s" does not exist, or an error occurred while getting the shared disk resource.') % {"name":disk_name})
            return OVM_FAIL
            
        self.log(sharedDiskResource._status)
        return [sharedDiskResource._status]
                        
    def do_sd_ls(self, options):
        """ sd_ls --- Get a list of Shared Virtual Disks """
        result = []

        serverpool_name = self._get_serverpool_name(options, silent=True)
        if serverpool_name is None:
            serverPools = self._get_all_serverpools()
        else:
            serverPools = self._get_serverpool_by_name(serverpool_name)
        
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        # Virtual_Disk_Name Server_Pool_Name Group_Name Size(MB) Status
        data = []
        data.append(('Virtual_Disk_Name', 'Server_Pool', 'Size(MB)', 'Group', 'Status'))          
        for sp in serverPools:
            shareddisks = self.rsm.getAllSharedVirtualDisks(sp._serverPoolName)
            for sd in shareddisks:
                group = self.adm.get_group_by_id(sd._groupId)
                if group is None:
                    group = ""
                size = sd._diskSize
                if self.OVM_VERSION != 'ovm215' and sd.__dict__.has_key('_deviceSize'):        
                    if size == 0 and sd._deviceSize is not None and sd._deviceSize != 0:
                        size = sd._deviceSize
                result.append((sp._serverPoolName, sd._diskName))
                data.append((sd._diskName, sp._serverPoolName, 
                      size, group, sd._status))
        if len(data) == 1:
            self.log(_('Shared virtual disk not found.'))
        else:
            self.log(indent(data, sep=self.sep))
        return result 

    def do_sd_disc(self, options):
        """ sd_disc --- List all discoverable shared virtual disk """
        
        if self.OVM_VERSION == 'ovm215':
            self._compatible_msg('C', '2.2')
            return OVM_FAIL
        
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR

        result = []
        suffix = '.img'
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
                
        # Virtual_Disk_Name Server_Pool_Name Group_Name Size(MB) Status
        data = []
        data.append(('Virtual_Disk_Name', 'Virtual_Disk_Type'))          
       
        multipath_devices = self.rsm.getUnregisteredMultipathDevices(serverpool_name)
        if multipath_devices is None:
            return OVM_FAIL
        shareddisks = self.rsm.getUnregisteredSharedVirtualDisks(serverpool_name)
        
        if multipath_devices is not None:
            for md in multipath_devices:
                diskname = md._fileName
                if diskname.find(suffix, -len(suffix)) > 0:
                    diskname = diskname[0:diskname.find(suffix, -len(suffix))]
                data.append((diskname, 'Multipath-based'))
        if shareddisks is not None:
            for sd in shareddisks:
                diskname = sd._fileName
                if diskname.find(suffix, -len(suffix)) > 0:
                    diskname = diskname[0:diskname.find(suffix, -len(suffix))]
                data.append((diskname, 'File-based'))

        if len(data) == 1:
            self.log(_('Discoverable shared virtual disk not found.'))
        else:
            self.log(indent(data, sep=self.sep))
        return result 


    def do_sd_reg(self, options):
        """ sd_reg --- Register a discoverable shared virtual disk"""
        if self.OVM_VERSION == 'ovm215':
            self._compatible_msg('C', '2.2')
            return OVM_FAIL
        
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        group = options.group
        multipath = options.multipath
        disk_name = options.disk_name
        comments = options.comments
        suffix = '.img'
        
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL

        groupid = self._get_groupid_by_name(group)
        if groupid is None:
            return OVM_FAIL
                        
        sd = None
        if multipath:
            multipath_devices = self.rsm.getUnregisteredMultipathDevices(serverpool_name)
            if multipath_devices is None:
                self._required_server('C', '2.2')
                return OVM_FAIL
            for md in multipath_devices:
                filename = md._fileName
                if filename.find(suffix, -len(suffix)) > 0:
                    filename = filename[0:filename.find(suffix, -len(suffix))]
                if disk_name == filename:
                    sd = md
                    break
            if not sd:
                self.log(_('Discoverable multipath-based shared virtual disk not found.'))
                return OVM_FAIL        
        else:
            shareddisks = self.rsm.getUnregisteredSharedVirtualDisks(serverpool_name)
            for shareddisk in shareddisks:
                filename = shareddisk._fileName
                if filename.find(suffix, -len(suffix)) > 0:
                    filename = filename[0:filename.find(suffix, -len(suffix))]
                    
                if disk_name == filename:
                    sd = shareddisk
                    break
            if not sd:
                self.log(_('Discoverable file-based shared virtual disk not found.'))
                return OVM_FAIL

        conf = self.datamodel.SharedVirtualDiskConfig()
        conf._diskName = disk_name
        if multipath:
            conf._deviceName = sd._deviceName
            conf._deviceSize = sd._deviceSize / 1000 / 1000
            conf._deviceID = sd._deviceUuid
        if comments:
            conf._comments = comments
        conf._userId          = self._get_login_userid(self.username)
        conf._groupId         = groupid
       
        result = False
        if multipath:
            result = self.rsm.registerMultipathDevice(serverpool_name, conf)              
        else:
            result = self.rsm.registerInternalSharedVirtualDisk(serverpool_name, conf)                        
        if result:
            self.log(_('Shared virtual disk "%(name)s" registered.' % {"name":disk_name}))
            return OVM_OK
        else:
            return OVM_FAIL
                      
    def do_svr_ls(self, options):
        """ svr_ls --- Get a list of VM Servers """

        serverpool_name = self._get_serverpool_name(options, silent = True)
        
        result = []
        if serverpool_name is None:
            serverPools = self._get_all_serverpools()
        else:
            serverPools = self._get_serverpool_by_name(serverpool_name)
            
        if serverPools is None:
            return OVM_NONE_SERVERPOOL
        # Server Host/IP  Server_Name Server_Type Status Location Server_Pool_Name          
        data = []
        data.append(('Server_Host/IP', 'Server_Name', 'Status', 'Server_Pool' ))
        for sp in serverPools:
            servers = self.spm.getAllMembers(sp)
            for server in servers:
                result.append((sp._serverPoolName, server._serverName))
                data.append(( server._ip, server._serverName, server._status, sp._serverPoolName))
        self.log(indent(data, sep=self.sep))
        return result

    def do_svr_restart(self, options):
        """ svr_restart --- Reboot a VM Server """
        
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        server_name      = options.server_name
        server = self._get_server_by_name(serverpool_name, server_name)
        if server is None:
            return OVM_NONE_SERVER          
        if self.spm.rebootServer(server):
            self.log(_('Rebooting server "%(name)s". ') % {"name":server_name})
            return OVM_OK
        else:
            return OVM_FAIL

    def do_svr_add(self, options):
        """ svr_add --- Add a Server to the ServerPool """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
                  
        hostname               = options.hostname
        server_name            = options.server_name
        agent_password         = options.agent_password
        server_location        = options.server_location
        description            = options.description
        utility_username       = options.utility_username
        utility_password       = options.utility_password       
        utility_only           = options.utility_only
        vm_only                = options.vm_only 
        configs = []

        if server_name is not None:
            if not self._check_sp_name_rules("Server Name", server_name):
                return OVM_OPTION_ERROR
            
        if vm_only and utility_only:
            self.log(_("vm_only and utility_only cannot both be specified."))
            return OVM_OPTION_ERROR
        
        if utility_only or not vm_only:
            if  utility_username is None:
                self.log(_("%s is required when adding a utility server.") % "option -U/--utility_username")
                return OVM_OPTION_ERROR
            if utility_password is None:
                utility_password = getpass(_("Utility server password: "))     
        
        if agent_password is None:
            agent_password = getpass(_("Oracle VM Agent password: "))
        
        if server_name is None:
            server_name = hostname            
        serverPools = self._get_serverpool_by_name(serverpool_name)
        if serverPools is None:
            return OVM_NONE_SERVERPOOL     
        sp = serverPools[0]
        if not utility_only:                 
            virtualServer   = self.datamodel.VirtualServer()
            # Form a virtualServer
            virtualServer._serverName = server_name 
            virtualServer._ip         = hostname
            virtualServer._agentPwd   = agent_password
            virtualServer._serverType = CONSTANTS.VIRTUAL_SERVER
            if server_location:
                virtualServer._location = server_location
            if description:
                virtualServer._comments = description                    
            serverConfig = self.datamodel.VirtualServerConfig()        
            serverConfig._virtualServer = virtualServer
            configs.append(serverConfig)
        if not vm_only:    
            # Form a utilityServer
            utilityServer   = self.datamodel.UtilityVirtualServer()
            utilityServer._ip         = hostname
            utilityServer._serverName = server_name
            utilityServer._agentPwd   = agent_password
            utilityServer._serverType = CONSTANTS.UTILITY_SERVER      
            utilityServer._loginUsername  = utility_username
            utilityServer._loginPassword   = utility_password
            if server_location:
                utilityServer._location = server_location
            if description:
                utilityServer._comments = description
            utilityServerConfig = self.datamodel.VirtualServerConfig()
            utilityServerConfig._utilityVirtualServer  = utilityServer
            configs.append(utilityServerConfig)
            
        if not self.spm.validateAddMembers(sp, configs):
            return OVM_FAIL
        
        if self.spm.addMembers(sp, configs):
            self.log(_("Added server \"%(name)s\" to the server pool.") %{"name":server_name})
            return OVM_OK
        else:
            return OVM_FAIL


    def do_svr_mode(self, options):
        """ svr_mode --- Set server maintenance or normal."""

        if self.OVM_VERSION == 'ovm215':
            self._compatible_msg('C', '2.2')
            return OVM_FAIL
                
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        server_name    = options.server_name
        maintain       = options.maintain
        normal         = options.normal
        
        if maintain and normal:
            self.log(_("maintain and normal cannot both be specified."))
            return OVM_OPTION_ERROR
        
        if (not maintain and not normal):
            self.log(_("Either maintain or normal should be specified."))
            return OVM_OPTION_ERROR
                
        server = self._get_server_by_name(serverpool_name, server_name)
        if server is None:
            return OVM_NONE_SERVER
        if maintain:
            if not self.spm.validateBlackoutServer(server._serverId):
                return OVM_FAIL
            if not self.spm.blackoutServer(server._serverId):
                return OVM_FAIL
            return OVM_OK
        if normal:
            if not self.spm.endBlackoutServer(server._serverId):
                return OVM_FAIL
            return OVM_OK
        
    def do_svr_del(self, options):
        """ svr_del --- Delete a Server from the ServerPool"""
        
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        server_name      = options.server_name
        force            = options.force
        server = self._get_server_by_name(serverpool_name, server_name)
        if server is None:
            return OVM_NONE_SERVER
        if self.spm.removeMember(server, force = force):
            self.log(_('Removed server \"%(name)s\" from the server pool.') % {"name":server_name})
            return OVM_OK
        else:
            return OVM_FAIL

    def do_svr_pwd(self, options):
        """ svr_pwd --- Change agent/utility server password"""

        if self.OVM_VERSION == 'ovm215':
            self._compatible_msg('C', '2.2')
            return OVM_FAIL
                
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        server_name      = options.server_name
        agent            = options.agent
        utility          = options.utility
        old_pwd          = options.old_pwd
        new_pwd          = options.new_pwd

        if agent and utility:
            self.log(_("agent and utility cannot both be specified."))
            return OVM_OPTION_ERROR
        
        if (not agent and not utility):
            self.log(_("Either agent or utility should be specified."))
            return OVM_OPTION_ERROR
        
        if agent or utility:
            if not old_pwd:
                print _("%s is required to change the password.") % "option -O/--old_pwd"
                return OVM_OPTION_ERROR
            if not new_pwd:
                print _("%s is required to change the password.") % "option -N/--new_pwd"
                return OVM_OPTION_ERROR
                        
        server = self._get_server_by_name(serverpool_name, server_name)
        if server is None:
            return OVM_NONE_SERVER
        if agent:
            if self.spm.validateUpdateVSAgentPassword(server._ip, old_pwd):
                if self.spm.updateVirtualServerAgentPassword(server._ip, old_pwd, new_pwd):
                    self.log(_('Agent password updated.'))
                    return OVM_OK
                else:
                    return OVM_FAIL
            else:
                return OVM_FAIL
        if utility:
            if self.spm.validateUpdateUtilityServerPassword(server._ip, old_pwd):
                    
                if self.spm.updateUtilityServerPassword(server._ip, old_pwd, new_pwd):
                    self.log(_('Utility server password updated.'))
                    return OVM_OK
                else:
                    return OVM_FAIL
            else:
                return OVM_FAIL

                                                
    def do_svr_poweroff(self, options):
        """ svr_poweroff --- Poweroff a VM Server """
        
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        server_name      = options.server_name
        
        server = self._get_server_by_name(serverpool_name, server_name)
        if server is None:
            return OVM_NONE_SERVER   
        if self.spm.shutdownServer(server):
            self.log(_('Shutting down server "%(name)s".') % {"name":server_name})
            return OVM_OK
        else:
            return OVM_FAIL                         

    def do_svr_info(self, options):
        """ svr_info --- Get a VM Server info """
        
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        server_name      = options.server_name
        
        server = self._get_server_by_name(serverpool_name, server_name)
        if server is None:
            return OVM_NONE_SERVER   
        metrics = self.spm.getVirtualServerMetricsById(server._serverId)
        numOfPhysicalCPU = ""
        cpuSpeedMHZ      = ""
        allocatedVCpu    = ""
        totalMemory      = ""
        freeMemory       = ""  
        if metrics is not None:
            if metrics != []:
                metrics = metrics[0]
                if metrics is not None:
                    numOfPhysicalCPU = metrics._numOfPhysicalCPU
                    cpuSpeedMHZ      = metrics._cpuSpeedMHZ
                    allocatedVCpu    = metrics._allocatedVCpu
                    totalMemory      = metrics._totalMemory
                    freeMemory       = metrics._freeMemory   
        location = server._location
        if location is None:
            location = "" 
        desc = server._comments
        if desc is None:
            desc = ""
        self.log('%18s %s' % ("ID:", server._serverId))
        self.log('%18s %s' % ("Server Host/IP:", server._ip))
        self.log('%18s %s' % ("Server Name:", server._serverName))
        self.log('%18s %s' % ("Server Type:", self._show_server_type(server._serverType)))
        self.log('%18s %s' % ("Status:", server._status))    
        self.log('%18s %s' % ("Server Pool:", serverpool_name))
        self.log('%18s %s' % ("Number of CPUs:", numOfPhysicalCPU))
        self.log('%18s %s' % ("CPU Speed(MHZ):", cpuSpeedMHZ))
        self.log('%18s %s' % ("Allocated VCPUs:", allocatedVCpu))
        self.log('%18s %s' % ("Total Memory(MB):", totalMemory))
        self.log('%18s %s' % ("Free Memory(MB):", freeMemory))
        self.log('%18s %s' % ("Location:", location))
        self.log('%18s %s' % ("Description:", desc))
        
        return OVM_OK

    def do_svr_stat(self, options):
        """ svr_stat --- Show the server status"""     
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        server_name      = options.server_name
        server = self._get_server_by_name(serverpool_name, server_name)
        if server is None:
            return OVM_NONE_SERVER
        self.log(server._status)
        return [server._status]
                
    def do_svr_conf(self, options):
        """ svr_conf --- Config a Virtual Server """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
                  
        server_name             = options.server_name
        new_name                = options.new_name
        agent_password          = options.agent_password
        username                = options.username
        password                = options.password
        server_type             = options.server_type
        description             = options.description
        location                = options.location
        force                   = options.force
        
        vm_server   = False
        util_server = False
        input_types = None
        server_types = []
        if server_type is not None:
            input_types = server_type.split(",")
            types = ['m', 'v', 'u']
            for type in input_types:
                if type not in types:
                    self.log(_("The server type \"%(name)s\" is invalid." %{"name":type}))
                    return OVM_OPTION_ERROR
                if type == 'v':
                    server_types.append(CONSTANTS.VIRTUAL_SERVER)
                    vm_server = True
                    continue
                if type == 'u':
                    server_types.append(CONSTANTS.UTILITY_SERVER)
                    util_server = True
                    continue
                if type == 'm':
                    server_types.append(CONSTANTS.MASTER_SERVER)
                    continue      

        if new_name is not None:
            if not self._check_sp_name_rules("New Server Name", new_name):
                return OVM_OPTION_ERROR
            
        if vm_server:
            if agent_password is None:
                print _("%s is required.") % "option -A/--agent_password"
                return OVM_OPTION_ERROR
        
        if util_server:
            if username is None:
                print _("%s is required.") % "option -u/--username"
                return OVM_OPTION_ERROR
            if password is None:
                password = getpass(_("Utility server password: "))
                
        server = self._get_server_by_name(serverpool_name, server_name)
        if server is None:
            return OVM_NONE_SERVER         
        updated = server
        if new_name is not None:
            updated._serverName      = new_name
        if username is not None:
            updated._loginUsername   = username
        if password is not None:
            updated._loginPassword   = password        
        if agent_password is not None:
            updated._agentPwd        = agent_password
        if server_type is not None:
            updated._serverType      = ",".join(server_types)
        if description is not None:
            updated._comments        = description
        if location is not None:
            updated._location        = location
        #print update.__dict__
        if self.spm.updateVirtualServer(updated, force = force):
            self.log(_("Server updated."))
            return OVM_OK
        else:
            return OVM_FAIL
        
    
    def do_svr_agent(self, options):
        """ svr_agent --- Get an agent version """
        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        server_name      = options.server_name
        
        server = self._get_server_by_name(serverpool_name, server_name)
        if server is None:
            return OVM_NONE_SERVER   
        version = self.spm.getAgentVersion(server._serverId)
        if version is None:
            self.log(_("Error occurred while getting the agent version."))
            return OVM_NONE_ERROR
        self.log(_("Agent version: %(version)s") % {"version":version})
        return [version]
            
    def do_user_new(self, options):
        """ user_new --- Create a User Account. """
        accountName = options.username
        password    = options.password
        email       = options.email
        firstName   = options.firstname
        lastName    = options.lastname
        locked      = options.disable
        role_name   = options.role_name
        RULES = re.compile("^[a-zA-Z_%][a-zA-Z0-9_-]*$")
        if not RULES.match(accountName):
            print _("Specified user account name is in invalid format.")
            return OVM_OPTION_ERROR
        EMAIL = re.compile("^[_.0-9a-z-]+@([0-9a-z][0-9a-z-]+.)+[a-z]{2,4}$")
        
        if not EMAIL.match(email):
            print _("Specified email address is in invalid format.")
            return OVM_OPTION_ERROR            
        
        if role_name is None:
            roleid = None
        else:
            roleid = self.adm.get_roleid_by_name(role_name)
        if roleid is None:
            if role_name is None:
                print _("%s is required.") % "option -r/--role_name"
            else:  
                self.log(_("Specified role does not exist."))
            self.log(_("Available roles:"))
            for role in self.adm.get_all_role_names():
                self.log(role)
            return OVM_FAIL
        
        if password is None:
            password = getpass(_("User password: "))
        if password is None or len(password) == 0:
            print _("User password is required and must not be NULL.")
            return OVM_OPTION_ERROR
            
        import md5, base64
        password = base64.encodestring(md5.new(password).digest()).strip()        
        if locked:
            accountStatus = CONSTANTS.USER_LOCKED
        else:
            accountStatus = CONSTANTS.USER_UNLOCKED                
        result  = self.adm.createUser(accountName, email, firstName, lastName, password, accountStatus, roleid)
        if result is None:
            return OVM_FAIL
        self.log(_("Created user \"%(name)s\".") % {"name":accountName})
        return OVM_OK

    def do_user_ls(self, options):
        """ user_ls --- List of all the users. """      
        user_configs = self.adm.findAllOVMUser()
        if user_configs is None:
            self.log(_("No user available, or an error occurred."))
            return OVM_FAIL
        data = []
        data.append(("Username",  "Group", "Server_Pool", "Role", "Status"))
        for user in user_configs:
            groups = self.adm.get_user_group_names(user._userId)
            roles  = self.adm.get_user_role_names(user._userId)
            if CONSTANTS.ADMINISTRATOR in roles:
                sites = ['All Server Pools']
            else:
                sites  = self._get_user_site_names(user._userId)
            data.append((user._accountName,  
                              ",".join(groups),
                              ",".join(sites), 
                              ",".join(roles), 
                              user._accountStatus))
        self.log(indent(data, sep=self.sep))
        return user_configs

    def do_group_ls(self, options):
        """ group_ls --- List of all the groups. """
        group_configs = self.adm.findAllOVMGroup()
        if group_configs is None:
            self.log(_("No group available, or an error occurred."))
            return OVM_FAIL
        data = []
        data.append(("Group",  "User_Count", "Description"))
        for group in group_configs:
            if group._groupId == -1 or group._groupId == 0:
                continue
            description = group._description
            number_of_user = self.adm.get_group_user_number(group._groupId)
            if description is None:
                description = ""
            data.append((group._groupName, number_of_user, description))
        if len(data) == 1:
            self.log(_('Group not found.'))
        else:
            self.log(indent(data, sep=self.sep))
        return group_configs
        
    def do_user_group_add(self, options):
        """ user_group_add --- Assign a user to the Group. """
        username = options.username
        groupname = options.groupname
        userid = self.adm.get_userid_by_name(username)
        if userid is None:
            self.log(_("User \"%(username)s\" does not exist." %{"username":username} ))
            return OVM_FAIL
        
        groupid = self._get_groupid_by_name(groupname)
        if groupid is None:
            return OVM_FAIL
        
        usergroup = self.adm.createUserGroup(userid, groupid)
        if usergroup is not None:
            self.log(_("User \"%(username)s\" has been assigned to the group." %{"username":username} )) 
            return OVM_OK
        else:
            return OVM_FAIL

    def do_user_svrp_add(self, options):
        """ user_svrp_add --- Assign a user to the ServerPool. """
        username  = options.username
        serverpool_name = options.serverpool_name
        
        userid = self.adm.get_userid_by_name(username)
        if userid is None:
            self.log(_("User \"%(username)s\" does not exist." %{"username":username} ))
            return OVM_FAIL
        
        serverpool = self._get_serverpool_by_name(serverpool_name)

        if serverpool is None:
            return OVM_FAIL
       
        usersite = self.adm.createUserSite(userid, serverpool[0]._serverPoolId)
        if usersite is not None:
            self.log(_("User \"%(username)s\" has been assigned to the server pool." %{"username":username} )) 
            return OVM_OK
        else:
            return OVM_FAIL

    def do_user_svrp_del(self, options):
        """ user_svrp_del --- Remove a user from the ServerPool. """
        username  = options.username
        serverpool_name = options.serverpool_name
        
        userid = self.adm.get_userid_by_name(username)
        if userid is None:
            self.log(_("User \"%(username)s\" does not exist." %{"username":username} ))
            return OVM_FAIL
        
        serverpool = self._get_serverpool_by_name(serverpool_name)

        if serverpool is None:
            return OVM_FAIL

        admin_ids  = self.adm.get_all_admin_ids()
        if userid in admin_ids:
            self.log(_("Operation not permitted on administrator users."))
            return OVM_FAIL
        
        org_ids    = self.adm.get_userids_by_siteid(serverpool[0]._serverPoolId)
        
        if userid not in org_ids:
            self.log(_("The specified user is not in the server pool."))
            return OVM_FAIL
        
        org_ids.remove(userid)            

        if self.spm.updateUserList4ServerPool(serverpool[0]._serverPoolId, org_ids):
            self.log(_("User \"%(username)s\" has been removed from the server pool." %{"username":username} )) 
            return OVM_OK
        else:
            return OVM_FAIL
            
    def do_group_new(self, options):
        """ group_new --- Create a User Group. """
        groupName = options.groupname
        desc    = options.desc
        RULES = re.compile("^[a-zA-Z_%][a-zA-Z0-9_-]*$")
        if not RULES.match(groupName):
            print _("The specified group name is in invalid format.")
            return OVM_OPTION_ERROR               
        result  = self.adm.createGroup(groupName, desc)
        if result is None:
            return OVM_FAIL
        self.log(_("Created user group \"%(name)s\".") % {"name":groupName})
        return OVM_OK

    def do_vncviewer(self, options):
        """ vncviewer --- Start a VNC console. """

        serverpool_name = self._get_serverpool_name(options)
        if serverpool_name is None:
            return OVM_OPTION_ERROR
        
        vm_name          = options.vm_name                  #   VM name
        vm = self._get_vm_by_name(serverpool_name, vm_name)
        if vm is None:
            return OVM_NONE_VM
        vncPort = vm._vncPort
        if vncPort is None:
            self.log((_("The virtual machine is not ready. Please try later.")))
            return OVM_NONE_ERROR
        vs = self.vmm.getAssociatedServer(vm)
        
        if vs is None:
            self.log((_("The virtual machine is not ready. Please try later.")))
            return OVM_NONE_SERVER
        server_host = vs._ip
        if server_host is None:
            self.log((_("Associated server IP address not found.")))
            return OVM_NONE_SERVER
        
        if os.path.exists(r"/usr/bin/vncviewer"):
            vncviewer = r"/usr/bin/vncviewer"
        else:
            ConfigFile = OvmConfigFile()   
            vncviewer = ConfigFile.getconfig("vncviewer")
            if vncviewer is None:
                return OVM_NONE_ERROR
            if len(vncviewer) == 0:
                ConfigFile.setconfig('vncviewer')
                return OVM_NONE_ERROR
        cmd = "%s %s:%s" %(vncviewer, server_host, vncPort)
        os.system(cmd)
    
    def do_use(self, options):
        """ use --- Specify a ServerPool to use"""
        serverpool_name = options.serverpool_name
        sp = self._get_serverpool_by_name(serverpool_name)
        if sp is not None:
             os.environ[CONSTANTS.SERVERPOOL_NAME] = serverpool_name
        else:
            return OVM_NONE_SERVERPOOL
        
    def do_connect(self, options):
        """ connect --- Connect to the Oracle VM Manager Server"""
        username = options.username
        password = options.password

        if username is None:
            username = getusername()
            if username == CONSTANTS.EOF:
                return 
        if password is None:
            password = getpass(_("Password:"))

        self._update_object(username=username, password=password)
        sp = self.spm.getAllServerPools()
        if sp is None:
            self.session = None
            return OVM_FAIL
        self.log(_("Connected."))

    def do_vm(self, options = None):
        self.do_help(['vm'])

    def do_vm_disk(self, options = None):
        self.do_help(['vm', 'disk'])

    def do_vm_nic(self, options = None):
        self.do_help(['vm', 'nic'])

    def do_vm_sd(self, options = None):
        self.do_help(['vm', 'sd'])

    def do_vm_cd(self, options = None):
        self.do_help(['vm', 'cd'])
                                
    def do_svr(self, options = None):
        self.do_help(['svr'])

    def do_svrp(self, options = None):
        self.do_help(['svrp'])
        
    def do_iso(self, options = None):
        self.do_help(['iso'])
        
    def do_tmpl(self, options = None):
        self.do_help(['tmpl'])
        
    def do_sd(self, options = None):
        self.do_help(['sd'])
        
    def do_img(self, options = None):
        self.do_help(['img'])

    def do_group(self, options = None):
        self.do_help(['group'])

    def do_user(self, options = None):
        self.do_help(['user'])

    def do_user_svrp(self, options = None):
        self.do_help(['user', 'svrp'])

    def do_user_group(self, options = None):
        self.do_help(['user', 'group'])
                
    def do_val(self, options = None):
        self.do_help(['val'])
                                                 
     
